pro bin_daily_raob_amsu
;=====================================================================
;@***h*  MW_Sounder_FCDR/bin_daily_raob_amsu.pro
;
; NAME
;        bin_daily_raob_amsu.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              For the radiosonde simulated AMSU BT data, this program 
;              loops over the specified range of Julian days and years, 
;              and reads in AMSU BT each day. The BTs are gridded and 
;              are written to an NetCDF file for later use.
;
; EXTERNALS:
;              none
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
; INPUTS:
;              Rebuilt NOAA brighness temperature:
;                $InPath_rebuild/
;
; OUTPUTS:
;              Converted NOAA MSU coefficients values to
;                $OutPath_noaa/
;
; AUTHOR:
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;           JD_BEG = The begining Julian day to process
;           JD_END = The ending Julian day to process
;             chan = MSU channel to process
;             noaa = Mission name to process
;   Inpath_rebuild = Path to rebuild pixel level data
; Inpath_MSU_MERRA = Path to MERRA simulated MSU BT
;Inpath_AMSU_MERRA = Path to MERRA simulated AMSU BT
;          OutPath = Path to output data
;
;              Path values should contain a trailing '/'
;**************************************************************************
            JD_BEG = julday(1,1,2001)
            JD_END = julday(1,1,2001)
              chan = 7
            Inpath = '../Data/tmp/simulate/raob_amsu/'
           Outpath = '../Data/tmp/simulate/raob_amsu_grid/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set Julian days that will be dealwithed
;-----------------------------------------------------------------------
jds = timegen(start=JD_BEG,final=JD_END,unit='day')

;
; Loop over the Julian days to process
;----------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set time variables
  ; -------------------------------------------
  jd = jds[ijd]
  caldat,jd,month,dom,year
  doy = julday(month,dom,year)-julday(1,0,year)
  syear = string(year,format='(i4.4)')
  smonth = string(month,format='(i2.2)')
  sdom = string(dom,format='(i2.2)')
  sdoy = string(doy,format='(i3.3)')

  ;
  ; Outfile
  ; ---------------------------------
  outfile = Outpath+syear+sdoy+'_nc'
  if file_test(outfile) then goto,pro_end

  ;--------------------------------------------
  ; read in gps 
  infile_gps=Inpath+'AMSU_ds351_Bt_'+syear+sdoy+'.dat'
  print,infile_gps
  if ~file_test(infile_gps) then goto,next_day
  lin = file_lines(infile_gps)
  if lin lt 3 then goto,next_day
  i = -1.
  num = 50000L
  year = fltarr(num)
  doy  = fltarr(num)
  sec  = fltarr(num)
  lat  = fltarr(num)
  lon  = fltarr(num)
  ratp = fltarr(num)
  tbs  = fltarr(15)
  bt   = fltarr(num)
  valid_max = fltarr(num)
  percentage = fltarr(num)
  close,2 & openr,2,infile_gps
  i = -1.0
  while ~eof(2) do begin
    i = i + 1
    data_0 = fltarr(24)
    readf,2,data_0
    year [i] = data_0[0]
    doy  [i] = data_0[1]
    sec  [i] = data_0[2]*3600.
    lat  [i] = data_0[3]
    lon  [i] = data_0[4]
    bt   [i] = data_0[chan+4]
    ratp [i] = data_0[20]
    level_num = data_0[23]

    data_1=fltarr(3,level_num)
    data_2=fltarr(20,100)
    readf,2,data_1
    readf,2,data_2

    chan_wf=data_2(4+chan,*)
    max_id = where(chan_wf eq max(chan_wf))
    pressure_in_max = data_2(1,max_id(0))
    valid_id=where(data_1(1,*) gt pressure_in_max*0.8 and data_1(1,*) lt pressure_in_max/0.8,valid_num)
    valid_max(i)=valid_num

    id_valid = where(data_2[4,*] eq 1)
    num_valid = total(data_2[4+chan,id_valid])
    num_all = total(data_2[4+chan,*])
    percentage[i] = num_valid/float(num_all)
  endwhile

  ;--------------------------------------------
  ;bin data
  ratp_ref = [52,61,79,80,81]
  dm_ratp = ratp*0
  for irt = 0,n_elements(ratp_ref)-1 do begin
    dm_ratp = dm_ratp or (ratp eq ratp_ref[irt])
  endfor
  id = where(year gt 1000 and lat ge -90 and lon ge -180 and lat le 90 and lon le 180 and bt gt 10 and dm_ratp and valid_max ge 3 and percentage gt 0.95)

  if n_elements(id) le 10 then goto,pro_end
  lat = lat[id]
  lon = lon[id]
  bt  = bt[id]

  ilat = fix((lat+90.0)/2.5) < 71
  ilon = fix((lon+180.0)/2.5) < 143

  ni =  ilon+ilat*144
  sni = sort(ni)
  ni = ni[sni]
  ui = uniq(ni)
  ni_ui = ni[ui]
  num = fltarr(144,72)
  if n_elements(ui) eq 1 then begin
    num[ni_ui] = n_elements(ni)
  endif else begin
    tnum = [ui[0]+1,ui[1:n_elements(ui)-1]-ui[0:n_elements(ui)-2]]
    num[ni_ui] = tnum
  endelse

  vl = double(bt[sni])
  vl_csum  = total(vl,/CUMULATIVE)
  vl_csum_ui = vl_csum[ui]
  if n_elements(ui) eq 1 then begin
    vl_mean = total(vl)/n_elements(vl)
  endif else begin
    vl_sum = [vl_csum_ui[0],vl_csum_ui[1:n_elements(ui)-1]-vl_csum_ui[0:n_elements(ui)-2]]
    vl_mean = vl_sum/tnum
  endelse
  mval = dblarr(144,72)-999
  mval[ni_ui] = vl_mean

  bt_bin = mval
  num_bin = num

  ;--------------------------------------------
  ;write data
  var = ['bt_bin','num_bin']
  ptrarr,dims,3
  ptrarr,ptr_var,3
  for i = 0,n_elements(var)-1 do begin
    *dims[i] = ['lon','lat']
    rt = execute('*ptr_var[i] = '+var[i])
  endfor
  outdir = file_dirname(outfile)
  if ~file_test(outfile) then spawn,'mkdir -p '+outdir
  
  fid = NCDF_CREATE(outfile,/CLOBBER)
  lat_dim = NCDF_DIMDEF(fid, 'lat', 72)
  lon_dim = NCDF_DIMDEF(fid,'lon', 144)
  NCDF_CONTROL, fid, /FILL
  vid = lonarr(n_elements(var))
  for iv = 0,n_elements(var)-1 do begin
    type = size(*ptr_var[iv],/tname)
    rt=execute('vid[iv] = NCDF_VARDEF(fid, var[iv], [lon_dim,lat_dim],/'+type+')');
  endfor
  NCDF_CONTROL, fid, /ENDEF
  for iv = 0,n_elements(var)-1 do begin
    NCDF_VARPUT, fid, vid[iv], *ptr_var[iv]
  endfor
  NCDF_CLOSE, fid ; Close the netCDF file.

  print,'O.K.'
  next_day:
endfor

pro_end:
stop
end
