pro convert_amsu_byraob
;=====================================================================
;@***h*  MW_Sounder_FCDR/convert_amsu_byraob.pro
; 
; NAME
;        convert_amsu_byraob.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the specified AMSU dataset, Read in RAOB correction
;              coeffcients from the specfied file, read in gridded AMSU
;              data, apply the correction, and write the converted data
;              to the given output file.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Monthly linear fit coefs between NOAA and RAOB data
;              are read from:
;                $InPath_coef/timeseries_offset_slope_{noaa}_raob_NH_TROP.dat
;                $InPath_coef/timeseries_offset_slope_{noaa}_raob_SH.dat
;
;              Gridded NOAA AMSU values are read from:
;                $InPath_noaa/YYYYMM/{noaa}_YYYYDDD_nc
;
; OUTPUTS:     
;              Converted NOAA AMSU values are written to the 
;              ASCCI file:
;                $OutPath/YYYYMM/{noaa}_YYYYDDD_nc
;
; AUTHOR:      
;              Liang Peng/ Ben Ho 
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;        noaa = The polar orbiter data to process.
;                 values={'noaa15', 'noaa16',
;                         'noaa18', 'noaa19', 'noaam2'}
; InPath_coef = Path to monthly fit coeffcients.
; InPath_noaa = Path to gridded NOAA data.
;OutPath_noaa = Path to converted NOAA data.
;
;              Path values should contain a trailing '/'
;**************************************************************************
        noaa='noaam2' 
 InPath_coef='../Data/tmp/linfit/offset_slope/'
 InPath_noaa='../Data/tmp/linfit/bin/'
OutPath_noaa='../Data/tmp/linfit/daily/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Construct the input coefficient file name for north hemisphere and tropical
;---------------------------------------------------------------------------
infile_coeff=InPath_coef+'timeseries_offset_slope_'+noaa+'_raob_NH_TROP.dat'

; Open coefficient file , allocate space 
; and read in coeffcients. 
;-----------------------------------------
openr,lun2,infile_coeff,ERROR=err,/get_lun
if(err ne 0) then begin
  print,' ERROR opening coeffcient file: err=',err
  print,' infile_coef=',infile_coef
  STOP
endif else begin
  num=file_lines(infile_coeff)
  temp=fltarr(4,num)
  readf,lun2,temp
  close,lun2
  free_lun,lun2
endelse
 
; Extract the desired values
;----------------------------
year_NH_TROP      =fix(temp[0,*])
month_NH_TROP     =fix(temp[1,*])
offset_NH_TROP    =temp[2,*] 
slope_NH_TROP     =temp[3,*]

;
; Construct the input coefficient file name for south hemishphere
;----------------------------------------------------------------
infile_coeff=InPath_coef+'timeseries_offset_slope_'+noaa+'_raob_SH.dat'

; Open coefficient file , allocate space 
; and read in coeffcients. 
;-----------------------------------------
openr,lun2,infile_coeff,ERROR=err,/get_lun
if(err ne 0) then begin
  print,' ERROR opening coeffcient file: err=',err
  print,' infile_coef=',infile_coef
  STOP
endif else begin
  num=file_lines(infile_coeff)
  temp=fltarr(4,num)
  readf,lun2,temp
  close,lun2
  free_lun,lun2
endelse
 
; Extract the desired values
;----------------------------
year_SH      =fix(temp[0,*])
month_SH     =fix(temp[1,*])
offset_SH    =temp[2,*] 
slope_SH     =temp[3,*]

; Start and ending of Julian day
;----------------------------
JD_BEG = julday(month_SH[0],1,year_SH[0])
JD_END = julday(month_SH[num-1]+1,0,year_SH[num-1])

; Loop Julian day
;---------------------------
for JD_loop = JD_BEG, JD_END do begin

  ; Time variables
  ;---------------------------
  caldat,JD_loop,month_loop,dom_loop,year_loop

  ; Read daily gridded data
  ;--------------------------
  infile = Inpath+syear+smonth+'/'+syear+sdoy+'_'+noaa+'_nc'
  print,infile
  if ~file_test(infile) then goto,pro_end
  var = ['chn_bin','num']
  ncid = NCDF_OPEN(infile,/NOWRITE)
  for iv = 0,n_elements(var)-1 do begin
    vd = NCDF_VARID(ncid, var[iv])
    rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv])
  endfor
  NCDF_CLOSE, ncid

  ; Calibrate BT by slope and offset
  ;-------------------------
  lat = findgen(72)*2.5-180+1.25
  id_NH_TROP = where(lat gt -20)
  id_SH      = where(lat lt -20)

  k_NH_TROP = where(month_loop eq month_NH_TROP and year_loop eq year_NH_TROP)
  k_SH      = where(month_loop eq month_SH      and year_loop eq year_SH)
  chn_bin[*,id_NH_TROP] = chn_bin[*,id_NH_TROP]*slope_NH_TROP[k_NH_TROP]+offset_NH_TROP[k_NH_TROP]
  chn_bin[*,id_SH] = chn_bin[*,id_SH]*slope_SH[k_SH]+offset_SH[k_SH]
  id = where(num eq 0)
  chn_bin[id] = -999

  ; Output data
  ;-------------------------
  ptr_var = ptrarr(2,/allocate_heap)
  *ptr_var[0] = chn_bin
  *ptr_var[1] = num
  outfile = Outpath+syear+smonth+'/'+syear+sdoy+'_'+noaa+'_nc'
  print,outfile
  var = ['chn_bin','num']
  outdir = file_dirname(outfile)
  if ~file_test(outdir) then spawn,'mkdir -p '+outdir
  fid = NCDF_CREATE(outfile,/CLOBBER)
  lat_dim = NCDF_DIMDEF(fid, 'lat', 72)
  lon_dim = NCDF_DIMDEF(fid,'lon', 144)
  NCDF_CONTROL, fid, /FILL
  vid = lonarr(n_elements(var))
  for iv = 0,n_elements(var)-1 do begin
    type = size(*ptr_var[iv],/tname)
    rt=execute('vid[iv] = NCDF_VARDEF(fid, var[iv], [lon_dim,lat_dim],/'+type+')');
  endfor
  NCDF_CONTROL, fid, /ENDEF
  for iv = 0,n_elements(var)-1 do begin
    NCDF_VARPUT, fid, vid[iv], *ptr_var[iv]
  endfor
  NCDF_CLOSE, fid ; Close the netCDF file.

endfor

;
; End Program
;--------------
end
