pro extract_amsu_bt
;=====================================================================
;@***h*  MW_Sounder_FCDR/extract_amsu_bt.pro
; 
; NAME
;        extract_amsu_bt.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the Specified NOAA polar orbiter 'noaa1', this program 
;              loops over the specified range of months and years, and reads 
;              the L1B binary files from the specified input directory. 
;              (downloaded from NOAA-CLASS TOVS from from KLM 8.3.1.6.1)
;              Values of brightness temperature and related data are stored 
;              for each julian day in ASCII files for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              L1B files should be stored with the following directory
;              structure and file format:
;
;                $InPath/{noaa1}/YYYYMM/NSS.AMAX.SS.Dyyddd.Snnnn.Ennnn.CC
;
;
; OUTPUTS:     
;              Output ASCII files are stored with the following directory 
;              structure and file format:
;
;                $OutPath/{noaa1}/YYYY/amsu_{noaa1}_YYYY.DDD.dat
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;                noaa1 = The name of the polar orbiter data to process.
;                        values = {'noaa15', 'noaa16', 'noaa18', 
;                                            'noaa19', 'noaam2'}
;   YEAR_BEG,MONTH_BEG = The begining month/year to process
;   YEAR_END,MONTH_END = The ending month/year to process
;   InPath             = The root path to the 'noaa1' L1B files.
;   OutPath            = The root path for the ASCII file to be generated.
;
;                       Path values should contain a trailing '/'
;**************************************************************************
    noaa1='noaa15'  
 YEAR_BEG=2001
MONTH_BEG=12
 YEAR_END=2014
MONTH_END=9
   InPath='/pub1/user/xinjiaz/msu_data/noaa_amsu/'
  OutPath='../extract/'
;*********************************************************
; End of RUN parameters
;*********************************************************

; 
; Set the appropriate 'noaa2' value for 'noaa1'
;  this value is the 2-character code for each NOAA orbiter
;     noaa_allcode=['NA','NC','NE','NF','NG','NH','ND', $
;                   'NI','NJ','NK','NL','NM','NN','NP']
;     noaa_code=noaa_allcode(noaa-6)  
;               noaa series starts from noaa-A(6)
;----------------------------------------------------------
case noaa1 of
  'noaa15': noaa2='NK'       
  'noaa16': noaa2='NL'       
  'noaa18': noaa2='NN'       
  'noaa19': noaa2='NP'       
  'noaam2': noaa2='M2'       
  else    : begin
              print,' ERROR: Unknown noaa1 value, noaa1=',noaa1
              STOP
            end
endcase

 ;
 ;initial variables for removing the 
 ;redundent data between two sequatial granules
 ;-----------------------------------
 maxnum_scanline = 1500  ;800 scanlines per day in general 
 year_previous = intarr(maxnum_scanline,2)
 day_previous = intarr(maxnum_scanline,2)
 sec_previous = ulonarr(maxnum_scanline,2)

;
; Loop over the years to process
;----------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the starting and ending month for the current year
  ;---------------------------------------------------------
  monthstart=1
  monthend=12
  if(year_loop eq YEAR_BEG) then monthstart=MONTH_BEG
  if(year_loop eq YEAR_END) then   monthend=MONTH_END

  ; Set the Julian day for the begining of each month
  ; and the number of days in the month
  ;--------------------------------------------------
  if((year_loop eq 2004) or $
     (year_loop eq 2008) or $
     (year_loop eq 2012)    ) then begin
   month_long  =[31,29,31,30,31,30,31,31,30,31,30,31]
   month_start =[1,32,61,92,122,153,183,214,245,275,306,336]
  endif else begin
   month_long  =[31,28,31,30,31,30,31,31,30,31,30,31]
   month_start =[1,32,60,91,121,152,182,213,244,274,305,335]
  endelse

  ; Loop over months
  ;---------------------
  for month_loop=monthstart,monthend do begin

    ; Loop over the days in the current month
    ;------------------------------------------
    jbeg=month_start[month_loop-1]
    jend=month_start[month_loop-1]+month_long[month_loop-1]-1
    for day_loop=jbeg,jend do begin
      print,year_loop,month_loop,day_loop

      ; Allocate space/variables
      ;--------------------------
      dummy1   =0B
      dummy2   =0U
      dummy4   =bytarr(4)
      dummy6   =bytarr(6)
      dummy8   =bytarr(8)
      dummy12  =bytarr(12)
      dummy16  =bytarr(16)
      dummy32  =bytarr(32)
      dummy40  =bytarr(40)
      dummy64  =bytarr(64)
      dummy268 =bytarr(268)
      dummy384 =bytarr(384)
      dummy336 =bytarr(336)
      dummy556 =bytarr(556)
      dummy864 =bytarr(864)
      dummy1628=bytarr(1628)

      dataset_id='aaa'
      format_version=uint(0)
      format_version_year=uint(0)
      format_version_day =uint(0)
      count_record=uint(0)
      dataset_name='123456789012345678901234567890123456789012'
      spacecraft_id=uint(0)
      data_type =uint(0)
      start_day = uint(0)     
      start_year = uint(0)     
      start_time = ulong(0)     
      end_day = uint(0)     
      end_year = uint(0)     
      end_time = ulong(0)     
       
      instrument_id=uintarr(3)

      rfshelf=intarr(12)

      rfmux=intarr(12)

      target_temperature=intarr(4,15)
      target_rfshelf=intarr(3,6)

      t_r_central_wave_number = lonarr(3,15)
      epoch_year=0U
      epoch_day=0U
      epoch_time=0UL
      semi_axis=0L
      eccentricity=0L
      inclination=0L
      argument_of_perigee=0L
      right_ascension=0L
      mean_anomaly=0L

      rfshelf_coefficient_a11=lonarr(4)
      rfshelf_coefficient_a12=lonarr(4)
      rfshelf_coefficient_a2=lonarr(4)
      warmload_coefficient_a11=lonarr(4,5)
      warmload_coefficient_a12=lonarr(4,5)
      warmload_coefficient_a2=lonarr(4,7)
    

      ; Construct path and file name structure, 
      ; then obtain a list of file to process.
      ;----------------------------------------------------
      indir  =InPath+string(year_loop ,format='(i4.4)')           $
                    +string(month_loop,format='(i2.2)')+'/'+noaa1+'/'        
      infiles='NSS.AMAX.'+noaa2+'.D'+string(year_loop-2000,format='(i2.2)') $
                                    +string(day_loop      ,format='(i3.3)')+'*'
      file_id = file_search(indir,infiles, count = icount)

      ; If there are no files in this day, goto to next day.
      ;-----------------------------------------------------
      if(icount eq 0) then begin
      print,indir,infiles
      goto,NEXT_DAY 
      endif

      ; Construct output file name and open it.
      ;------------------------------------------
      outfile=OutPath+noaa1+'/'+string(year_loop ,format='(i4.4)')     $
            +'/amsu_'+noaa1+'_'+string(year_loop ,format='(i4.4)')+'.' $
                               +string(day_loop  ,format='(i3.3)')     $
                               +'.dat'
      err=file_test(OutPath,/DIRECTORY,/WRITE)
      if(err ne 1) then begin
       print,' OutPath does not exist or does not have write permission'
       print,' OutPath=',OutPath
       STOP
      endif
      outdir=file_dirname(outfile)
      err=file_test(outdir,/DIRECTORY,/WRITE)
      if(err ne 1 ) then begin
       file_mkdir,outdir
      endif
      openw,3,outfile,ERROR=err
      if(err ne 0) then begin
       print,' ERROR opening output file: err=',err
       print,' outfile=',outfile
       STOP
      endif

      ; Loop over the files for the current day
      ;--------------------------------------------
      for file_loop=0,icount-1 do begin

        ; set current scanline as previous scanline
        ; for removing the redundent data 
        ;----------------------------------------------------
        year_previous(*,0) = year_previous(*,1)
        day_previous(*,0) = day_previous(*,1)
        sec_previous(*,0) = sec_previous(*,1)
        ii_scanline = 0

        ; open input file
        ;-----------------------
        infile=file_id[file_loop]
        openr,2,infile,/SWAP_IF_LITTLE_ENDIAN

        ;read the title information, 
        ;--------------------------------
        readu,2,dataset_id 
        if(dataset_id eq 'NOT') then goto,NEXT_FILE

        readu,2,dummy1, format_version, format_version_year,               $
          format_version_day, dummy12, dataset_name, dummy8,               $
          spacecraft_id,dummy2, data_type, dummy6, start_year, start_day,  $
          start_time, dummy4, end_year, end_day, end_time, dummy40,        $
          count_record, dummy64, instrument_id, rfshelf,rfmux,             $
          target_temperature, target_rfshelf, dummy268,                    $
          t_r_central_wave_number, dummy32, epoch_year, epoch_day,         $
          epoch_time, semi_axis,eccentricity, inclination,                 $
          argument_of_perigee, right_ascension, mean_anomaly, dummy556,    $
          rfshelf_coefficient_a11, rfshelf_coefficient_a12, dummy16,       $
          warmload_coefficient_a11, warmload_coefficient_a12, dummy384,    $
          rfshelf_coefficient_a2, dummy16, warmload_coefficient_a2, dummy336
 
        ; The scan line num in one round ,generally there are 800 scan 
        ; lines in one round, and there are about 10 round 
        ; (one file contain one round) everyday
        ;-------------------------------------------------------------------
        for record_loop=0,count_record-1 do begin 

          ; Allocate type/size of values
          ;------------------------------
          dummy10=bytarr(10)
          dummy12=bytarr(12)
          dummy52=bytarr(52)
          dummy60=bytarr(60)
          dummy76=bytarr(76)
          dummy116=bytarr(116)
          dummy128=bytarr(128)
          dummy212=bytarr(212)
          dummy268=bytarr(268)
          number_of_scan=0U
          year_of_scan=0U
          day_of_scan=0U
          clockdrift_of_scan=0
          time_of_day_of_scan=0UL
          bitfield_of_scan   =0U
          quality_indicator_bit_field=0UL
	  scan_line_quality_flags1=0B
	  scan_line_quality_flags2=0B
	  scan_line_quality_flags3=0B
	  scan_line_quality_flags4=0B
	  Calibration_Quality_Flags=uintarr(16)
          pri_cal_coeffs=lonarr(3,15)
          angular_relationships=intarr(3,30)
          earth_location=lonarr(2,30)
          scene_telemetry_a1=uintarr(510)
          Temperature_Sensor_Telemetry_a1=uintarr(46)
          scene_telemetry_a2=uintarr(120)
          Temperature_Sensor_Telemetry_a2=uintarr(20)

          ; read the received electon count:scene_telemetry_a1 and 
          ; scene_telemetry_a2 and read other parameters: lat,lon etc
          ;-------------------------------------------------------------
          readu,2,number_of_scan, year_of_scan, day_of_scan,               $
            clockdrift_of_scan, time_of_day_of_scan, bitfield_of_scan,     $
            dummy10, quality_indicator_bit_field,                          $
	    scan_line_quality_flags1, scan_line_quality_flags2,            $
	    scan_line_quality_flags3, scan_line_quality_flags4,            $
	    Calibration_Quality_Flags, dummy16, pri_cal_coeffs,            $
            dummy212, angular_relationships, earth_location, dummy12,      $
            scene_telemetry_a1, dummy60, Temperature_Sensor_Telemetry_a1,  $
            dummy116, scene_telemetry_a2, dummy12,                         $
            Temperature_Sensor_Telemetry_a2, dummy76            

          ; Process good scanlines
          ;-------------------------
          if(quality_indicator_bit_field eq 0) then begin 

            ;remove the redundent data 
            ;------------------------------------
            overlap_id  =  where(year_previous(*,0) eq year_of_scan and $
                                 day_previous(*,0)  eq day_of_scan  and $
                                 sec_previous(*,0)  eq time_of_day_of_scan, overlap_num)
            if overlap_num ne 0 then begin
               goto,jump_overlap
            endif
            year_previous(ii_scanline,1) = year_of_scan
            day_previous(ii_scanline,1)  = day_of_scan
            sec_previous(ii_scanline,1)  = time_of_day_of_scan
            ii_scanline = ii_scanline+1

            ; Loop over FOV.
            ; there are 30 fov in one scanline
            ;------------------------------------
            lat=fltarr(30)
            lon=fltarr(30)
            sza=fltarr(30)
            lza=fltarr(30)
            for ifov_loop=0,30-1 do begin

              lat[ifov_loop]=earth_location[0,ifov_loop]*1.0E-4
              lon[ifov_loop]=earth_location[1,ifov_loop]*1.0E-4

	      ; remove bad lat-lon pixel
	      ;------------------------------------
	      if abs(lat[ifov_loop]) gt 90 or abs(lon[ifov_loop]) gt 180 then begin
	          print,'bad lat-lon'
                  goto,jump_badlatlon
	      endif

              sza[ifov_loop]=angular_relationships[0,ifov_loop]*1.0E-2
              if(ifov_loop lt 15) then begin
                lza[ifov_loop]=-1*angular_relationships[1,ifov_loop]*1.0E-2
              endif else begin
                lza[ifov_loop]=angular_relationships[1,ifov_loop]*1.0E-2
              endelse

              ; Calculate the bright temperature in these 
              ; all 15 channels in AMSU1
              ;  (we are only interested for chan7~12)
              ;------------------------------------------------
              at=dblarr(15)
              at[*]=-99. 
              for chan_loop=(1-1),(15-1) do begin  

	        ;if flag is bad, then don't use it 
		;-------------------------------------------------------
		if Calibration_Quality_Flags(chan_loop) ne 0 then begin
		   goto,jump_badchan
		endif

                ; Set the count for this channel
                ; chan0,1 belong to AMSU-2 & chan3~15 belong to AMSU-1
                ;-------------------------------------------------------
                if(chan_loop lt 2) then begin  
                  count = scene_telemetry_a2[ifov_loop*4 +chan_loop+2]
                endif else begin
                  count = scene_telemetry_a1[ifov_loop*17+chan_loop+2]
                endelse

                ; Convert from count to radiance
                ;-----------------------------------------
                clcoef2=pri_cal_coeffs[0,chan_loop]*1.0E-19
                clcoef1=pri_cal_coeffs[1,chan_loop]*1.0E-13
                clcoef0=pri_cal_coeffs[2,chan_loop]*1.0E-09
                rad=clcoef0 + (clcoef1*count) + (clcoef2*count*count)

                ; Now convert from radiance to 
                ; antennae temperature(brightness temperature)
                ;-----------------------------------------------
                plcoef1=float(1.191042869E-05)
                plcoef2=float(1.438777)

                temp=0.0D
                pl=float(t_r_central_wave_number[0,chan_loop]*1E-6)

                c1v3=plcoef1*pl*pl*pl
                c2v =plcoef2*pl
 
                ; If radiance > 0 , then tbs is valid
                ;------------------------------------
                if(rad ne 0) then begin
                  temp=double(c1v3/rad)
                  if(temp gt 0) then begin
                    temp=alog(temp + 1.0)
                    at[chan_loop]=(c2v/temp)
                  endif
                endif 

		; goto next channel 
		;---------------------------- 
		jump_badchan:
                 
              endfor ; chan_loop=1-1,15-1 

              ; Save values for later use.
              ;----------------------------
              printf,3,year_of_scan,day_of_scan,double(time_of_day_of_scan)/1000. ,$
                       lat[ifov_loop],lon[ifov_loop],sza[ifov_loop],lza[ifov_loop],$
                       at,                                                         $
              format='(i4,2x,i3,1x,3(f10.3,1x),2(f9.2,1x),15(f10.3,1x))'

	      ; goto next pixel
	      ;------------------- 
	      jump_badlatlon:

            endfor ; ifov_loop=0,30-1 

            ; goto next scanline
	    ;-------------------
	    jump_overlap:

          endif ; (quality_indicator_bit_field eq 0) 
        endfor  ; record_loop=0,count_record-1 

        ; goto target for next file
        ;----------------------------
        NEXT_FILE:

        ; close input file.
        ;-------------------
        close,2
      endfor ; file_loop=0,icount-1 

      ; goto target, when there are no files in this day
      ;------------------------------------------------
      NEXT_DAY:

      ; close output file.
      ;--------------------
      close,3
    endfor ; day_loop=jbeg,jend 
  endfor ; month_loop=monthstart,monthend 
endfor ; year_loop=YEAR_BEG,YEAR_END 
;
; End Program
;----------------
end
