pro extract_msu_coef
;=====================================================================
;@***h*  MW_Sounder_FCDR/extract_msu_coef.pro
;
; NAME
;        extract_msu_coef.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              For the Specified NOAA polar orbiter 'noaa1', this program
;              loops over the specified range of months and years, and 
;              reads the L1B binary files from the specified input directory.
;              (downloaded from NOAA-CLASS TOVS from NOAA MSU) Values of 
;              counts and related data are stored for each julian day in 
;              NetCDF files for later use.
;
;
; EXTERNALS:
;              none
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
;
; INPUTS:
;              NOAA 1b binary data:
;                $InPath/
;
; OUTPUTS:
;              NOAA coefficient data:
;                $OutPath_noaa/
;
; AUTHOR:
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;           JD_BEG = The begining Julian day to process
;           JD_END = The ending Julian day to process
;             noaa = Mission name to process
;           Inpath = Path to NOAA binary data
;          OutPath = Path to NOAA coefficient data
;
;              Path values should contain a trailing '/'
;**************************************************************************
   JD_BEG = julday(1,1,2001)
   JD_END = julday(1,1,2001)
     noaa = 'noaa14'
   Inpath = '../Data/Input/msu/'
  Outpath = '../Data/tmp/extract/msu_coef/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Define the central wave number (unit is cm-1)
;-----------------------------------------------
if noaa eq 'noaa14' or $
   noaa eq 'noaa11' or $
   noaa eq 'noaa10' or $
   noaa eq 'noaa9'  or $
   noaa eq 'noaa8' then pl = [1.6779, 1.7927, 1.8334, 1.9331]
if noaa eq 'noaa12' then pl = [1.6799, 1.7927, 1.8334, 1.9331]
if noaa eq 'noaa7'  then pl = [1.6779, 1.7927, 1.8337, 1.9331]
if noaa eq 'noaa6' or $
   noaa eq 'tiros' then pl = [1.6779, 1.7927, 1.8337, 1.9331]

;
; Set Julian days that will be dealwithed
;------------------------------------------------
jds = timegen(start=JD_BEG,final=JD_END,unit='day')

;
; Loop over Julian days to process
;----------------------------------
for ijd = 0,n_elements(jds)-1 do begin
  jd = jds[ijd]
  caldat,jd,month,dom,year
  outfile = outpath+str_jd(jd,'yyyymm')+'/'+str_jd(jd,'yyyyddd')+'_msu_'+noaa+'_nc'
  print,outfile
  if file_test(outfile) then goto,next_day

  ;
  ; Set initial values
  ; ---------------------------------
  varout = ['year/1','doy/1','sec/1','alt/1','lza/1','lat/11','lon/11','sza/11',$
	 'digital/28','count/56','coef/24','scan_count/28','flag/20','analog/14',$
         'bt01/11','bt02/11','bt03/11','bt04/11']
  numscan = 20000L
  kk = -1.;
  for iv = 0,n_elements(varout)-1 do begin
    vv = varout[iv]
    vv = strsplit(vv,'/',/extract)
    if double(vv[1]) eq 1 then begin
      rt = execute(''+vv[0]+'_out=fltarr(numscan)')
    endif else begin
      rt = execute(''+vv[0]+'_out=fltarr('+vv[1]+',numscan)')
    endelse
  endfor

  ;
  ; Loop over daily Julian days to process
  ;----------------------------------
  doy = julday(month,dom,year)-julday(1,0,year)
  syear = string(year,format='(i4.4)')
  smonth = string(month,format='(i2.2)')
  sdom = string(dom,format='(i2.2)')
  sdoy = string(doy,format='(i3.3)')

  ;   
  ; Set input files
  ;----------------------------------
  infiles = file_search(Inpath+syear+smonth+'/'+noaa+'/NSS.????.??.D'+str_jd(jd,'yyddd')+'*',count=ninf)
  if ninf eq 0 then goto,next_day
  for ifile = 0,n_elements(infiles)-1 do begin
    infile = infiles[ifile]
    print,infile

    num = 437l
    nchan = 4.
    nfov = 11.
    if year lt 1995 then num = 440l

    ii_bad1=0L
    ii_bad2=0L
    ii_bad3=0L

    ;
    ; Decide if contain archive header
    ; ------------------------------------
    ah = 0
    close,2
    openr,2,infile,/SWAP_IF_LITTLE_ENDIAN
    td = 'aaaaaa'
    readu,2,td
    if td eq 'NOTAPP' then ah=1
    close,2

    ;
    ; Read archive header
    ; ------------------------------------
    openr,2,infile,/SWAP_IF_LITTLE_ENDIAN
    if ah eq 1 then begin
      td = bytarr(512)
      readu,2,td
    endif

    ;
    ; Read title
    ; ------------------------------------
    vars = ['count_record=intarr(1)/9/10']
    n2 = 0
    for iv = 0,n_elements(vars)-1 do begin
      var = strsplit(vars[iv],'/',/extract)
      n1 = long(var[1])
      if n1-n2 gt 1 then begin
	td = bytarr(n1-n2-1)
	readu,2,td
      endif
      n2 = long(var[2])
      rt = execute(var[0])
      varnm = strsplit(var[0],'=',/extract)
      rt = execute('readu,2,'+varnm[0])
    endfor
    if num-n2 gt 0 then begin
      td = bytarr(num-n2)
      readu,2,td
    endif

    ;
    ; Loop over scan record
    ;----------------------------------
    for iscan = 0,count_record[0]-1 do begin

      ;
      ; read dataset 
      ; ---------------------------------
      vars = ['scan_line = intarr(1)/1/2',$
	      'time_code_year_day = uintarr(1)/3/4',$
	      'time_code_millisec = ulonarr(1)/5/8',$
	      'scan_quality_indicator = bytarr(4)/9/12',$
	      'Earth_Location_Delta = lonarr(1)/13/16',$
	      'Calibration_Coefficients = lonarr(2,4)/17/48',$
	      'Calibration_Coefficients_normalization = lonarr(4,4)/49/112',$
	      'Height_and_Local_Zenith_Angle = intarr(2)/113/116',$
	      'Earth_Location_Data = intarr(2,11)/117/160',$
	      'MSU_Data = intarr(112)/161/384',$
	      'Scan_Position_Quality = bytarr(16)/385/400'$
	     ]
	      
      n2 = 0
      for iv = 0,n_elements(vars)-1 do begin
	var = strsplit(vars[iv],'/',/extract)
	n1 = long(var[1])
	if n1-n2 gt 1 then begin
	  td = bytarr(n1-n2-1)
	  readu,2,td
	endif
	n2 = long(var[2])
	rt = execute(var[0])
	varnm = strsplit(var[0],'=',/extract)
	rt = execute('readu,2,'+varnm[0])
      endfor
      if num-n2 gt 0 then begin
	td = bytarr(num-n2)
	readu,2,td
      endif

      ;
      ; Remove the bad record
      ; -----------------------------
      if scan_quality_indicator(0) ne 0 or $
	 scan_quality_indicator(2) ne 0 or $
	 (scan_quality_indicator(1) and 'FB'x) ne 0  then begin ;bad scanline
	ii_bad1=ii_bad1+1L
	goto,next_scan
      endif

      ;
      ; Convert to year and day
      ; -------------------------------
      time_code_year = (time_code_year_day and 'FE00'x)/2L^9  ;7 bits in msb
      if time_code_year lt 70 then begin
	 time_code_year = time_code_year+2000
      endif else begin
	 time_code_year = time_code_year+1900
      endelse
      time_code_day  = (time_code_year_day and '01FF'x)       ;9 bits in lsb

      ;
      ; Satellite locations
      ; --------------------------------
      lat = Earth_Location_Data[0,*]/128.
      lon = Earth_Location_Data[1,*]/128.
      sza = lat
      for ilat = 0,10 do begin
	lonlat2solz,lon[ilat],lat[ilat],time_code_year,time_code_day,double(time_code_millisec/1000.),sza1,sola1
	sza[ilat] = sza1
      endfor
      lza = Height_and_Local_Zenith_Angle[1]
      alt = Height_and_Local_Zenith_Angle[0]

      ;
      ; Testing the unvalid lat-lon which pass the quality control
      ; ---------------------------------------------
      if scan_quality_indicator(0) eq 0            and $
	 scan_quality_indicator(2) eq 0            and $
	(scan_quality_indicator(1) and 'FB'x) eq 0 and $
	 (abs(lat[5]) gt 90 or abs(lon[5]) gt 180) then begin
	 year_day_lat_lon =[time_code_year,time_code_day,lat[5],lon[5]]
	 print,'bad flag1',year_day_lat_lon
	 goto,next_scan
      endif

      analog  = lonarr(14)
      digital = lonarr(2,14)
      raw_count  = lonarr(4,14)
      scan_count = lonarr(2,14)
      jj=0
      for ii = 0,104,8 do begin
	 ; --- all these output below extracted from MSU_Data ---
	 ; http://www.ncdc.noaa.gov/oa/pod-guide/ncdc/docs/podug/html/c4/sec4-3.htm
	 ; Table 4.3.2.1-4. Format of MSU data.
	 analog(jj)=MSU_Data(ii)   and '0FFF'x      ; the first column is instrument voltage (analog), '0FFF' is 0000 1111 1111 1111, Table 4.3.2.1-5. Typical format for all MSU words (except scan position-line count).
	 digital(0,jj)=MSU_Data(ii+1) and '0FFF'x   ; the second column is prt count  (digital)
	 digital(1,jj)=MSU_Data(ii+2) and '0FFF'x   ; the third column is prt count too (digital)
	 raw_count(0,jj)=MSU_Data(ii+3) and '0FFF'x  ;chan1 , (include scene count, cold count, warm count and SCAN TO SPOT )
	 raw_count(1,jj)=MSU_Data(ii+4) and '0FFF'x  ;chan2
	 raw_count(2,jj)=MSU_Data(ii+5) and '0FFF'x  ;chan3
	 raw_count(3,jj)=MSU_Data(ii+6) and '0FFF'x  ;chan4
	 scan_count(0,jj)=MSU_Data(ii+7) and '00FF'x  ;scan angle data
	 scan_count(1,jj)=(MSU_Data(ii+7) and '0700'x)/2^8  ;scan line count
	 ; --- extract end ----
	 jj=jj+1
      endfor

      ;
      ; Avoid the overflow output
      ; ---------------------------------------
      if time_code_millisec/1000. gt 999999 then time_code_millisec = double(999999)*1E3
      unvalid_id = where(Calibration_Coefficients_normalization gt 9999999999. or Calibration_Coefficients_normalization lt -999999999.,unvalid_num)
      if unvalid_num gt 0 then Calibration_Coefficients_normalization(unvalid_id)=9999999999
      unvalid_id = where(Calibration_Coefficients gt 9999999999. or Calibration_Coefficients lt -999999999.,unvalid_num)
      if unvalid_num gt 0 then Calibration_Coefficients(unvalid_id)=9999999999

      ; 
      ; Get brightness temperature
      ; ---------------------------------------
      for chan_loop=1-1,4-1 do begin
	;
	; Transfer from raw count to normalized count
	; ----------------------------------------------
	rad_count = double(MSU_Data(chan_loop+3:chan_loop+3+8*11-1:8) and '0FFF'x)
 
	;
	; Generate radiance
	; ---------------------------------------------
	normalized_count = Calibration_Coefficients_normalization(0,chan_loop)            *double(2.^(-22))+$
			   Calibration_Coefficients_normalization(1,chan_loop)*rad_count  *double(2.^(-30))+$
			   Calibration_Coefficients_normalization(2,chan_loop)*rad_count^2*double(2.^(-44))+$
			   Calibration_Coefficients_normalization(3,chan_loop)*rad_count^3*double(2.^(-56))

	;
	; Transfer from normalized count to radiance
	; ---------------------------------------------
	rad = Calibration_Coefficients(1,chan_loop)                  *double(2.^(-22))+$
	      Calibration_Coefficients(0,chan_loop)*normalized_count *double(2.^(-30))
        bt = bstemp(rad,pl[chan_loop])
        rt=execute('bt'+string(chan_loop+1,f='(i2.2)')+'=bt')
      endfor

      ; 
      ; Combin data
      ; ---------------------------------------
      kk = kk+1.0
      year_out[kk] = time_code_year
      doy_out[kk] = time_code_day
      sec_out[kk] = double(time_code_millisec)/1000.
      alt_out[kk] = alt
      lza_out[kk] = lza
      lat_out[*,kk] = lat
      lon_out[*,kk] = lon
      sza_out[*,kk] = sza
      digital_out[*,kk] = digital
      count_out[*,kk] = raw_count
      coef_out[ 0:15,kk] = Calibration_Coefficients_normalization(*,0:3)
      coef_out[16:23,kk] = Calibration_Coefficients(*,0:3)
      scan_count_out[*,kk] = scan_count
      flag_out[*,kk] = [scan_quality_indicator,Scan_Position_Quality]
      analog_out[*,kk] = analog
      bt01_out[*,kk] = bt01
      bt02_out[*,kk] = bt02
      bt03_out[*,kk] = bt03
      bt04_out[*,kk] = bt04

      next_scan:
    endfor ;scan
    close,2
  endfor ; file

  ;
  ; Output data
  ; -----------------------------------------
  if kk le 0 then goto,pro_end
  ptrarr,ptr_var,n_elements(varout)
  ptrarr,dims,n_elements(varout)

  for iv = 0,n_elements(varout)-1 do begin
    vv = strsplit(varout[iv],'/',/extract)
    v = vv[0]
    rt=execute('*ptr_var[iv] = '+v+'_out')
    rt=execute('ndim = size('+v+'_out,/N_DIMENSIONS)')
    if ndim gt 1 then begin
      rt=execute('*ptr_var[iv] = '+v+'_out[*,0:kk]')
    endif else begin
      rt=execute('*ptr_var[iv] = '+v+'_out[0:kk]')
    endelse

    if strmatch(v,'*/11') then begin
      *dims[iv] = ['scan','time']
    endif else if ndim gt 1 then begin
      *dims[iv] = [v,'time']
    endif else begin
      *dims[iv] = 'time'
    endelse

    varout[iv] = v
  endfor

  outdir = file_dirname(outfile)
  if ~file_test(outdir) then spawn,'mkdir -p '+outdir
  fid = NCDF_CREATE(outfile,/CLOBBER)
  time_dim = NCDF_DIMDEF(fid, 'time', n_elements(*ptr_var[0]))
  NCDF_CONTROL, fid, /FILL
  vid = lonarr(n_elements(varout))
  for iv = 0,n_elements(varout)-1 do begin
    type = size(*ptr_var[iv],/tname)
    vv = strsplit(varout[iv],'/',/extract)
    vv = 'tt_'+vv[0]
    rt=execute('vid[iv] = NCDF_VARDEF(fid, vv, time_dim,/'+type+')');
  endfor
  NCDF_CONTROL, fid, /ENDEF
  for iv = 0,n_elements(varout)-1 do begin
    NCDF_VARPUT, fid, vid[iv], *ptr_var[iv]
  endfor
  NCDF_CLOSE, fid ; Close the netCDF file.

  next_day:
endfor ; data

pro_end:
end 
