pro merra_day_nc 
;=====================================================================
;@***h*  MW_Sounder_FCDR/merra_day_nc.pro
;
; NAME
;        merra_day_nc.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              For the radiosonde simulated AMSU BT data, this program 
;              loops over the specified range of Julian days and years, 
;              and reads in AMSU BT each day. The BTs are gridded and 
;              are written to an NetCDF file for later use.
;
;
; EXTERNALS:
;              none.
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
;
; INPUTS:
;              MERRA AMSU/MSU simalated ASCII data:
;                $InPath_coef/
;
; OUTPUTS:
;              
;              IDL format data:
;                $OutPath/YYYYDDD.sav
;
; AUTHOR:
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;       JD_BEG = The begining Julian day to process
;       JD_END = The ending Julian day to process
;          lza = Satellite zenith angle
;         chan = channel ID
;         type = AMSU or MSU
;       InPath = Path to simulated MERRA simulated BT
;      OutPath = Path to gridded simulated MERRA simulated BT
;
;              Path values should contain a trailing '/'
;**************************************************************************
JD_BEG = julday(1,1,1979)
JD_END = julday(1,1,1979)
lza  = '0'
chan = 4
type = 'msu'
 Inpath = '../Data/tmp/simulate/'
Outpath = '../Data/tmp/simulate/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set Julian days that will be dealwithed
; -------------------------------------------
jds = timegen(start=jd_beg,final=jd_end,unit='day')

;
; Set string of channel
; -------------------------------------------
schan = string(chan,f='(i2.2)')

;
; Loop over the Julian days to process
;--------------------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set time variables
  ; -------------------------------------------
  jd = jds[ijd]
  caldat,jd,month,dom,year
  doy = julday(month,dom,year)-julday(1,0,year)
  syear = string(year,format='(i4.4)')
  smonth = string(month,format='(i2.2)')
  sdom = string(dom,format='(i2.2)')
  sdoy = string(doy,format='(i3.3)')

  ;
  ; Set input file name
  ; -------------------------------------------
  infile = Inpath+'merra_'+type+'/'+syear+sdoy+'.lza'+lza+'.txt'
  if (~file_test(infile)) then goto,next_jd

  ;
  ; Set output file name
  ; -------------------------------------------
  outfile = Outpath+'merra_'+type+'_grid/'+syear+sdoy+'.lza'+lza+'_nc'

  ;
  ; Read in data
  ; -------------------------------------------
  if type eq 'msu' then begin
    col = 9
  endif else begin
    col = 20
  endelse
  close,2 
  openr,2,infile
  lin = file_lines(infile)
  data = fltarr(col,lin)
  readf,2,data
  tb = data[4+chan,*]
  close,2
  bt = fltarr(72,36,4)
  bt[*] = tb
  
  ;
  ; Output data
  ; -------------------------------------------
  fid = NCDF_CREATE(outfile,/CLOBBER)
  lat_dim  = NCDF_DIMDEF(fid,'lat', 36)
  lon_dim  = NCDF_DIMDEF(fid,'lon', 72)
  hour_dim = NCDF_DIMDEF(fid,'hour',4 )
  NCDF_CONTROL, fid, /FILL
  vid = NCDF_VARDEF(fid, 'bt', [lon_dim,lat_dim,hour_dim])
  NCDF_CONTROL, fid, /ENDEF
  NCDF_VARPUT, fid, vid, bt
  NCDF_CLOSE, fid ; Close the netCDF file.

  next_jd:
endfor

pro_end:
stop
end
