pro offset_slope_raob_noaa_month
;=====================================================================
;@***h*  MW_Sounder_FCDR/offset_slope_raob_noaa_month.pro
;  
; NAME
;        offset_slope_raob_noaa_month.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For each 1 month or 3 months running in the specified
;              range, read in the matched AMSU temperatures between 
;              the specified polar orbiter and for all specified raob
;              instrument types. Calcualte the linear fit coefficients 
;              for each month. Save the resulting monthly values in an
;              ASCII file for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Matched AMSU values are read from the ASCII files with the
;              following directory structure and file format:
;
;                $InPath/{noaa}_raob/match_{noaa}_raob_YYYYDDD.dat
;
; OUTPUTS:     
;              Linear fit coefficients are written to the ASCII file:
;
;                $OutPath/timeseries_offset_slope_{noaa}_raob_1monthrunning.dat
;                $OutPath/timeseries_offset_slope_{noaa}_raob_3monthrunning.dat
;
; AUTHOR:      
;              Xinjia Zhou / Liang Peng / Ben Ho 
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;                noaa = The name of the NOAA satellite to process.
;                           values = {'noaa15', 'noaa16', 'noaa18',
;                                     'noaa19', 'noaam2'}
;  YEAR_BEG,MONTH_BEG = The begining year and Month to process
;  YEAR_END,MONTH_END = The ending year and Month to process
;  RUNNING_MONTH      = The number of months for the running mean 
;   InPath            = The root path to the NOAA-RAOB match files.
;   OutPath           = The root path for the ASCII slope and offset 
;                       files to be generated.
;
;                       Path values should contain a trailing '/'
;
; Matching criteria for colocating NOAA and RAOB data, these values can 
; strengthen the Critea that were used when generating the match files.
;
;        CRI_time = TIME Criteria
;        CRI_dist = DISTANCE Criteria
;        CRI_sang = SATELLITE VIEWING ANGLE Criteria
;        CRI_bt   = BRIGHTNESS TEMPERATURE Criteria
;      CRI_ratp   = specific raob instrument types
;      CRI_level  = RAOB level Criteria 1
;      CRI_cover  = RAOB level Criteria 2
;
;**************************************************************************
     noaa='noaa15' 
 YEAR_BEG=2001
MONTH_BEG=1
 YEAR_END=2014
MONTH_END=9
RUNNING_MONTH = 1   
 InPath  ='../Data/ASCII_tmp/match/'
 OutPath ='../Data/ASCII_tmp/offset_slope/'
 CRI_time=1.0  ;                    TIME Criteria units = hours
 CRI_dist=100. ;                DISTANCE Criteria units = km
 CRI_sang=15.  ; SATELLITE VIEWING ANGLE Criteria units = angle
   CRI_bt=5.   ;  BRIGHTNESS TEMPERATURE Criteria units = degrees
CRI_ratp1=52   ;  RS80 57H
CRI_ratp2=61   ;  RS80 Loran/DigiCore I II or Marwin
CRI_ratp3=79   ;  RS92 DigiCore I II OR Marwin
CRI_ratp4=80   ;  RS92 DigiCore III
CRI_ratp5=81   ;  RS92 Autosonde
CRI_level=3.   ;  Criteria for observation number in AMSU ch7 weighting function maximum height range
               ;  unit = level number
CRI_cover=0.95 ;  Criteria for weighting percentage covered by real ROAB observation
               ;  unit = percentage

;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Construct the output file name and open it
;---------------------------------------------
outfile=OutPath+'timeseries_offset_slope_'+noaa+'_raob_'+string(RUNNING_MONTH,format='(i1)')+'month_running.dat'
err=file_test(OutPath,/DIRECTORY,/WRITE)
if(err ne 1) then begin
  print,' OutPath does not exist or does not have write permission'
  print,' OutPath=',OutPath
  STOP
endif
outdir=file_dirname(outfile)
err=file_test(outdir,/DIRECTORY,/WRITE)
if(err ne 1 ) then begin
  file_mkdir,outdir
endif
openw,lun2,outfile,ERROR=err,/get_lun
if(err ne 0) then begin
  print,' ERROR opening output file: err=',err
  print,' outfile=',outfile
  STOP
endif


MAX_NUM=5000000L
;
; Loop over the years to process
;-------------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the starting and ending month for the current year
  ;---------------------------------------------------------
  monthstart=1
  monthend=12
  if(year_loop eq YEAR_BEG) then monthstart=MONTH_BEG
  if(year_loop eq YEAR_END) then   monthend=MONTH_END

  ; Loop over months
  ;--------------------
  for month_loop=monthstart,monthend do begin

    ; Allocate spcae/variables
    ;--------------------------
    year      =intarr(MAX_NUM) 
    month     =intarr(MAX_NUM)
    day       =intarr(MAX_NUM)
    time_noaa =fltarr(MAX_NUM)
    time_raob =fltarr(MAX_NUM)
    lat_noaa  =fltarr(MAX_NUM)
    lon_noaa  =fltarr(MAX_NUM)
    lat_raob  =fltarr(MAX_NUM)
    lon_raob  =fltarr(MAX_NUM)
    sza       =fltarr(MAX_NUM)
    sat       =fltarr(MAX_NUM)
    cloud     =fltarr(MAX_NUM)
    ratp      =fltarr(MAX_NUM)
    corn      =fltarr(MAX_NUM)
    sirc      =fltarr(MAX_NUM)
    valid_max =fltarr(MAX_NUM)
    percentage=fltarr(MAX_NUM)
    bt_noaa   =fltarr(MAX_NUM)
    bt_raob   =fltarr(MAX_NUM)
    inum=0L

    ; Loop over the running months
    ;-------------------------------------------
    for running_loop=-1*(RUNNING_MONTH-1)/2,(RUNNING_MONTH-1)/2 do begin
 
        ; Loop over the days in the current month running
        ;-------------------------------------------
        caldat,julday(month_loop+running_loop  ,1,year_loop), month_running,day_temp,year_running
        jbeg = julday(month_loop+running_loop  ,1,year_loop)-julday(1 ,1,year_running)+1
        jend = julday(month_loop+running_loop+1,0,year_loop)-julday(1 ,1,year_running)+1

        for day_loop=jbeg,jend do begin    

          ; Construct the input file name 
          ;----------------------------------
          infile=InPath+noaa+'_raob'+                         $
                  '/'+string(year_running,format='(i4.4)')    $
                    +'/match_'+noaa+'_raob_'                  $
                    +string(year_running,format='(i4.4)')+'.' $
                    +string( day_loop,format='(i3.3)')+'.dat'
          ;print,infile

          ; Open the RAOB dataset for this day and read in data
          ;---------------------------------------------------------
          openr,lun1,infile,error=err,/get_lun
          if(err ne 0) then begin
            print,'open file err',infile
            goto,NEXT_DAY_err
          endif else begin
            ; If there are not enough profiles, then goto the next 
            ; day, otherwise allocate space and read in data. 
            ;---------------------------------------------------------
            total_num=file_lines(infile)
            if(total_num eq 0) then begin
              print,'total_num eq 0',infile
              goto,NEXT_DAY_empty
            endif 
            data_total=fltarr(38,total_num)
            readf,lun1,data_total
          endelse
  
          ; Extract the desired values
          ;------------------------------
          year     [inum:(inum+total_num-1)]  = year_running
          month    [inum:(inum+total_num-1)]  = month_running
          day      [inum:(inum+total_num-1)]  =data_total[ 1,*]
          time_noaa[inum:(inum+total_num-1)]  =data_total[ 2,*]
          time_raob [inum:(inum+total_num-1)]  =data_total[ 3,*]
          lat_noaa [inum:(inum+total_num-1)]  =data_total[ 4,*]
          lon_noaa [inum:(inum+total_num-1)]  =data_total[ 5,*]
          lat_raob  [inum:(inum+total_num-1)]  =data_total[ 6,*]
          lon_raob  [inum:(inum+total_num-1)]  =data_total[ 7,*]
          cloud    [inum:(inum+total_num-1)]  =data_total[ 8,*]
          ratp     [inum:(inum+total_num-1)]  =data_total[ 9,*]
          corn     [inum:(inum+total_num-1)]  =data_total[10,*]
          sirc     [inum:(inum+total_num-1)]  =data_total[11,*]
          sza      [inum:(inum+total_num-1)]  =data_total[12,*]
          sat      [inum:(inum+total_num-1)]  =data_total[13,*]
          bt_noaa  [inum:(inum+total_num-1)]  =data_total[14,*] ;chan7
          bt_raob   [inum:(inum+total_num-1)]  =data_total[15,*]
          valid_max [inum:(inum+total_num-1)] =data_total[26,*] 
          percentage[inum:(inum+total_num-1)] =data_total[32,*] 

  
          ; Increment the number of matches
          ;---------------------------------
          inum=inum+long(total_num)
          print,'inum=',inum

          ; goto target for the next GPS misson
          ;-------------------------------------
          NEXT_DAY_empty:
          close,lun1
          free_lun,lun1
          NEXT_DAY_err:
        endfor  ;  day_loop=jbeg,jend 
    endfor ; running_loop = -1*(RUNNING_MONTH-1)/2,(RUNNING_MONTH-1)/2

    ; If there are not enough matches, got the next month
    ;-----------------------------------------------------
    if(inum lt 5) then goto,NEXT_MONTH

    ; Trim array sizes
    ;-------------------
    day      =      day[0:(inum-1)]
    time_noaa=time_noaa[0:(inum-1)]
    time_raob = time_raob[0:(inum-1)]
    lat_noaa = lat_noaa[0:(inum-1)]
    lon_noaa = lon_noaa[0:(inum-1)]
    lat_raob =  lat_raob[0:(inum-1)]
    lon_raob =  lon_raob[0:(inum-1)]
    ratp     = ratp[0:(inum-1)]
    sza      =      sza[0:(inum-1)]
    sat      =      sat[0:(inum-1)]
    bt_noaa  =  bt_noaa[0:(inum-1)]
    bt_raob  =   bt_raob[0:(inum-1)]
    valid_max=valid_max[0:(inum-1)]
    percentage = percentage[0:(inum-1)]

    ; Allocate some space
    ;-----------------------

      ; Get indices of data points which 
      ; satisify the given criteria
      ;------------------------------------
      dif_time=abs(time_noaa-time_raob)
      dif_BT  =abs(bt_noaa-bt_raob)

      ;distance equation
      ;------------------------------------------
      radius = 6371  ;earth mean radius, km
      cos_distance = cos(lat_raob/180.*!pi)*cos(lat_noaa/180.*!pi)*$
                     cos(lon_raob/180.*!pi-lon_noaa/180.*!pi)+$
                     sin(lat_raob/180.*!pi)*sin(lat_noaa/180.*!pi)
      ;cos_distance might larger than 1 or less than -1, and lead arithmetic error: Floating illegal operand
      id_plus_1 =where(cos_distance gt 1, num_plus_1)
      id_minus_1=where(cos_distance lt -1, num_minus_1)
      if num_plus_1 gt 0  then cos_distance(id_plus_1)=1
      if num_minus_1 gt 0 then cos_distance(id_minus_1)=-1

      distance = radius*acos(cos_distance)
      ;distance equation end
      ;------------------------------------------
       
      id=where((  dif_BT lt CRI_bt  )  and $
               (dif_time lt CRI_time)  and $
               (distance lt CRI_dist)  and $
               (abs(sat) lt CRI_sang)  and $
                   (ratp eq CRI_ratp1  or  $
                    ratp eq CRI_ratp2  or  $
                    ratp eq CRI_ratp3  or  $
                    ratp eq CRI_ratp4  or  $
                    ratp eq CRI_ratp5) and $
               valid_max ge CRI_level  and $
              percentage gt CRI_cover      $
               ,ncount)
      print,'ncount=', ncount 

      ; If there are enough points, calculate 
      ; the slope and offset values.
      ;----------------------------------------
      if(ncount lt 2) then begin
        coeff=[-999.,-999.]
        mean_chan =-999.
        std_chan  =-999.
      endif else begin
        abt      =bt_noaa[id]
        bbt      =bt_raob [id]
        coeff    =poly_fit(abt,bbt,1,yfit)
        mean_chan=mean(bbt-abt)
        std_chan =stddev(abt-bbt)
      endelse
      num_chan=ncount

      ; Save the reuslts in the output file
      ;--------------------------------------
      if ncount ge 2 then begin
        printf,lun2,year_loop,month_loop,coeff,mean_chan,std_chan, mean(bbt),mean(abt),correlate(abt,bbt),num_chan,$
             format='(i4,2x,i2,2x,4(f12.3,2x),2(f12.3,2x),f12.4,6(2x,i6))'
       endif
    ; goto target for next month
    ;-----------------------------
    NEXT_MONTH:
  endfor ; month_loop=monthstart,monthend 
endfor ; year_loop=YEAR_BEG,YEAR_END
;
; Close Output file
;--------------------
close, lun2
free_lun,lun2
;
; End Program
;--------------
end 
