pro offset_slope_raob_noaa_month_modify
;=====================================================================
;@***h*  MW_Sounder_FCDR/offset_slope_raob_noaa_month_modify.pro
;  
; NAME
;        offset_slope_raob_noaa_month_modify.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              Because the radionsonde station are mostly located in
;              tropical and north hemisphere, modify and generate the
;              slope-offset coefs in two regions, 90N~20S and 20S~90S 
;              using the rules below:
;
;              In 90N~20S:
;                 always use 1 month slope and offset 
;
;              In 20S~90S, 
;                 if 1 month slope is between 0.95~1.05 then
;                   use 1 month slope and offset 
;                 else if 3 month slope is between 0.95~1.05 then
;                   use 3 month slope and offset 
;                 else if 3 month slope is lower than 0.95 
;                   slope  = 0.95-(0.95-slope_3month)/10 
;                   offset = mean(raob)-mean(noaa*slope)
;                 else if 3 month slope is larger than 1.05 
;                   slope  = 1.05+(slope_3month-1.05)/10
;                   offset = mean(raob)-mean(noaa*slope)
;                 endif
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Monthly linear fit coefs between NOAA and RAOB data
;              are read from:
;                $InPath_coef/timeseries_offset_slope_{noaa}_raob_1month_running.dat
;                $InPath_coef/timeseries_offset_slope_{noaa}_raob_3month_running.dat
;
;
; OUTPUTS:     
;              Converted NOAA AMSU values are written to the 
;              ASCCI file:
;               $outPath_coef/timeseries_offset_slope_{noaa}_raob_NH_TROP.dat
;               $outPath_coef/timeseries_offset_slope_{noaa}_raob_SH.dat
;
; AUTHOR:      
;              Xinjia Zhou / Liang Peng
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;        noaa = The polar orbiter data to process.
;                 values={'noaa15', 'noaa16',
;                         'noaa18', 'noaa19', 'noaam2'}
; InPath_coef = Path to monthly fit coefficients.
;OutPath_coef = Path to modified fit coefficients.
;
;              Path values should contain a trailing '/'
;**************************************************************************
        noaa='noaa19' 
 InPath_coef='../Data/ASCII_tmp/offset_slope/'
OutPath_coef='../Data/ASCII_tmp/offset_slope/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Construct the input coefficient file name
;--------------------------------------------
infile1_coeff=InPath_coef+'timeseries_offset_slope_'+noaa+'_raob_1month_running.dat'
infile2_coeff=InPath_coef+'timeseries_offset_slope_'+noaa+'_raob_3month_running.dat'

;
; Construct the output coefficient file name
;--------------------------------------------
outfile1_coeff=OutPath_coef+'timeseries_offset_slope_'+noaa+'_raob_NH_TROP.dat'
outfile2_coeff=OutPath_coef+'timeseries_offset_slope_'+noaa+'_raob_SH.dat'

; Open coefficient file , allocate space 
; and read in coeffcients. 
;-----------------------------------------
openr,lun1,infile1_coeff,ERROR=err,/get_lun
if(err ne 0) then begin
  print,' ERROR opening coeffcient file: err=',err
  print,' infile_coef=',infile1_coef
  STOP
endif else begin
  num1=file_lines(infile1_coeff)
  temp1=fltarr(10,num1)
  readf,lun1,temp1
  close,lun1
  free_lun,lun1
endelse
 
openr,lun2,infile2_coeff,ERROR=err,/get_lun
if(err ne 0) then begin
  print,' ERROR opening coeffcient file: err=',err
  print,' infile_coef=',infile2_coef
  STOP
endif else begin
  num2=file_lines(infile2_coeff)
  temp2=fltarr(10,num2)
  readf,lun2,temp2
  close,lun2
  free_lun,lun2
endelse

; Extract the desired values
;----------------------------
year1      =fix(temp1[0,*])
month1     =fix(temp1[1,*])
offset1    =temp1[2,*] 
slope1     =temp1[3,*]

year2      =fix(temp2[0,*])
month2     =fix(temp2[1,*])
offset2    =temp2[2,*] 
slope2     =temp2[3,*]
raob2      =temp2[6,*]
noaa2      =temp2[7,*]

; Open output file
;------------------------
openw,lun3,outfile1_coeff,ERROR=err,/get_lun
if(err ne 0) then begin
    print,' ERROR opening output NOAA file: err=',err
    print,' outfile=',outfile1_coeff
    STOP
endif

openw,lun4,outfile2_coeff,ERROR=err,/get_lun
if(err ne 0) then begin
    print,' ERROR opening output NOAA file: err=',err
    print,' outfile=',outfile2_coeff
    STOP
endif


; Loop over the months in the coef file
;-------------------------------------
for month_loop=0,(num1-1) do begin

  ;search the corresponding month in 3 month running list
  ;---------------------------------------------------------------------------------------
  month_id = where(year2 eq year1(month_loop) and month2 eq month1(month_loop), month_num)
  if month_num ne 1 then begin
    goto,jump_month
  endif

  ;calculate the slope and offset in south hemisphere
  ;--------------------------------------------------
  if slope1(month_loop) ge 0.95 and slope1(month_loop) le 1.05 then begin
    slope_sh =slope1(month_loop)
    offset_sh=offset1(month_loop)
  endif else if slope2(month_id(0)) ge 0.95 and slope2(month_id(0)) le 1.05 then begin
    slope_sh =slope2(month_id(0))
    offset_sh=offset2(month_id(0))
  endif else if slope2(month_id(0)) lt 0.95 then begin
    slope_sh=0.95-(0.95-slope2(month_id(0)))/10
    offset_sh=mean(raob2(month_id(0)))-mean(noaa2(month_id(0))*slope_sh) 
  endif else if slope2(month_id(0)) gt 1.05 then begin
    slope_sh  = 1.05+(slope2(month_id(0))-1.05)/10 
    offset_sh=mean(raob2(month_id(0)))-mean(noaa2(month_id(0))*slope_sh) 
  endif else begin
    stop
  endelse

  ; Loop over lines, write out modified coefs
  ;------------------------------------------
  printf,lun3,year1(month_loop),month1(month_loop),offset1(month_loop),slope1(month_loop),$
         format='(i4,2x,i2,2x,2(f12.3,2x))'

  printf,lun4,year1(month_loop),month1(month_loop),offset_sh,slope_sh,$
         format='(i4,2x,i2,2x,2(f12.3,2x))'

  ;Goto next month
  ;-----------------
  jump_month:

endfor ; month_loop=0,(num1-1) 

; Close output file
;---------------------
close,lun3
free_lun,lun3
close,lun4
free_lun,lun4

;
; End Program
;--------------
end
