pro sno_step1_preparematchup_msu
;=====================================================================
;@***h*  MW_Sounder_FCDR/sno_step1_preparematchup_msu.pro
;
; NAME
;        sno_step1_preparematchup_msu.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the MSU matched data, this program loops over the 
;              specified range of Julian days and years, and reads in 
;              MSU raw count written to an IDL data file for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Monthly matched NOAA MSU data:
;                $InPath_match/
;
;              Monthly NOAA coefficient data:
;                $InPath_coef/
;
; OUTPUTS:     
;              Converted matched NOAA coefficients to 
;              IDL format data:
;                $OutPath/step1_{noaa1}_{noaa2}.sav
;
; AUTHOR:      
;              Liang Peng / Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Liang Peng (Formatted for CDR)
;
; 
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;       JD_BEG = The begining Julian day to process
;       JD_END = The ending Julian day to process
;         chan = MSU channel to process
;        noaas = Mission pairs to process 
; InPath_coef  = Path to monthly fit coeffcients.
; InPath_match = Path to matched polar orbiter data
; OutPath      = Path to output data
;
;              Path values should contain a trailing '/'
;**************************************************************************
JD_BEG = julday(1,1,1978)
JD_END = julday(1,1,2006)
chan = 3
noaas = 'noaa12/noaa14'
InPath_coef  = '../Data/tmp/extract/msu_coef/'
InPath_match = '../Data/tmp/match/'
OutPath      = '../Data/tmp/sno/match_coef/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set string values for channel
;-----------------------------------------------------------------------
schan = string(chan,format='(i2.2)')

;
; Set Julian days that will be dealwithed
;-----------------------------------------------------------------------
jds = timegen(start=JD_BEG,final=JD_END,unit='month')

;
; Set satellite names for the match pair
;-----------------------------------------------------------------------
noaas = strsplit(noaas,'/',/extract)
noaa1 = noaas[0]
noaa2 = noaas[1]

;
; Set output file name
;----------------------------------
outfile =OutPath+'step1_'+noaa1+'_'+noaa2+'.sav'

;
; Loop over the Julian days to process
;----------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set the month and year
  ;----------------------------------
  jd = jds[ijd]
  caldat,jd,month_loop,dom_loop,year_loop
  print,year_loop,month_loop

  ;
  ; Set parameters for satellites
  ;----------------------------------
  msu_parameter,noaa1,pl_noaa1,k_noaa1,e_1a_noaa1,e_2a_noaa1,e_1b_noaa1,e_2b_noaa1
  msu_parameter,noaa2,pl_noaa2,k_noaa2,e_1a_noaa2,e_2a_noaa2,e_1b_noaa2,e_2b_noaa2

  ;
  ; Set input file name
  ;----------------------------------
  infile_match = InPath_match+noaa1+'_'+noaa2+'/match_'+noaa1+'_'+noaa2+'_'+string(year_loop,f='(i4.4)')+string(month_loop,f='(i2.2)')+'_nc'
  infile_coef1 = InPath_coef+string(year_loop,f='(i4.4)')+string(month_loop,f='(i2.2)')+'_msu_'+noaa1+'_nc'
  infile_coef2 = InPath_coef+string(year_loop,f='(i4.4)')+string(month_loop,f='(i2.2)')+'_msu_'+noaa2+'_nc'
  if ~(file_test(infile_match) and file_test(infile_coef1) and file_test(infile_coef2)) then goto,next_month
  
  ;
  ; Read the match data
  ;----------------------------------
  var = ['year','doy','sec_noaa1','sec_noaa2','lat_noaa1','lon_noaa1','lat_noaa2','lon_noaa2','sza_noaa1','sza_noaa2','lza_noaa1','lza_noaa2','bt'+schan+'_noaa1','bt'+schan+'_noaa2']
  ncid = NCDF_OPEN(infile_match,/NOWRITE)
  for iv = 0,n_elements(var)-1 do begin
    vd = NCDF_VARID(ncid, var[iv])
    rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv])
  endfor
  NCDF_CLOSE, ncid
  doy_match = doy
  rt = execute('bt_noaa1 = '+'bt'+schan+'_noaa1')
  rt = execute('bt_noaa2 = '+'bt'+schan+'_noaa2')

  ;
  ; Read MSU coef data
  ;----------------------------------
  var = ['doy','sec','count','digital','coef']
  for inoaa = 1,2 do begin
    snoaa = string(inoaa,format='(i1.1)')
    rt = execute('ncid = NCDF_OPEN(infile_coef'+snoaa+',/NOWRITE)')
    for iv = 0,n_elements(var)-1 do begin
      vd = NCDF_VARID(ncid, var[iv])
      rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv])
    endfor
    NCDF_CLOSE, ncid
    rt = execute('doy_'+snoaa+'=doy')
    rt = execute('sec_'+snoaa+'=sec')

    ;count
    rt=execute('count_scene_'+snoaa+'= count(19+chan,*)') ;nadir viewing position
    rt=execute('count_cold_'+snoaa+' = count(43+chan,*)') ;space viewing
    rt=execute('count_warm_'+snoaa+' = count(47+chan,*)') ;warm target 

    ;digital
    num_coef = n_elements(doy)
    rt=execute('bt_digital_'+snoaa+' = dblarr(2,num_coef) - 999.')
    if chan eq 1 or chan eq 2 then begin
      count_target  = digital(14:15,*) ; target1 (for ch1 and ch2)
      count_low     = digital(0,*)     ; low reference 
      count_high    = digital(2,*)     ; high reference 
      rt=execute('e_a_noaa'+snoaa+' = e_1a_noaa'+snoaa)
      rt=execute('e_b_noaa'+snoaa+' = e_1b_noaa'+snoaa)
    endif else if chan eq 3 or chan eq 4 then begin
      count_target  = digital(16:17,*) ; target2 (for ch3 and ch4)
      count_low     = digital(1,*)     ; low reference 
      count_high    = digital(3,*)    ; high reference 
      rt=execute('e_a_noaa'+snoaa+' = e_2a_noaa'+snoaa)
      rt=execute('e_b_noaa'+snoaa+' = e_2b_noaa'+snoaa)
    endif

    divident_zero_id = where(count_high ne count_low, divident_zero_num)
    if divident_zero_num eq 0 then goto,pro_end
    rt=execute('Ra_noaa'+snoaa+' = k_noaa'+snoaa+'(0) + k_noaa'+snoaa+'(1)*[count_target(0,divident_zero_id)-count_low(divident_zero_id)]/[count_high(divident_zero_id)-count_low(divident_zero_id)]')  ;1st warm scan
    rt=execute('Rb_noaa'+snoaa+' = k_noaa'+snoaa+'(0) + k_noaa'+snoaa+'(1)*[count_target(1,divident_zero_id)-count_low(divident_zero_id)]/[count_high(divident_zero_id)-count_low(divident_zero_id)]')  ;2nd warm scan

    rt=execute('bt_digital_'+snoaa+'(0,divident_zero_id) = Ra_noaa'+snoaa+' * e_a_noaa'+snoaa+'(2)^2 + Ra_noaa'+snoaa+' * e_a_noaa'+snoaa+'(1) + e_a_noaa'+snoaa+'(0)')
    rt=execute('bt_digital_'+snoaa+'(1,divident_zero_id) = Rb_noaa'+snoaa+' * e_b_noaa'+snoaa+'(2)^2 + Rb_noaa'+snoaa+' * e_b_noaa'+snoaa+'(1) + e_b_noaa'+snoaa+'(0)')

    ;coef
    if chan eq 1 then begin
    ; Calibration_Coefficients_normalization = coef(0:3,*)     ;ch1 and ch2 are not available yet
    endif else if chan eq 2 then begin
    ; Calibration_Coefficients_normalization = coef(0:3,*)     
    endif else if chan eq 3 then begin
     rt=execute('Calibration_Coefficients_normalization_'+snoaa+' = coef(8:11,*)')
     rt=execute('Calibration_Coefficients_'+snoaa+' = coef(20:21,*)')
    endif else if chan eq 4 then begin
     rt=execute('Calibration_Coefficients_normalization_'+snoaa+' = coef(12:15,*)')
     rt=execute('Calibration_Coefficients_'+snoaa+' = coef(22:23,*)')
    endif
  endfor

  ;
  ; find the SNO match pairs
  ;----------------------------------
  ; criterias 
  cri_dist= 111    ;unit is km
  cri_time  = 120    ;unit is sec
  cri_bt  = 100
  cri_sat = 1      ;only choose the nadir viewing position
  cri_count = 0

  dif_time = abs(sec_noaa1-sec_noaa2)
  dif_BT   = abs(bt_noaa1-bt_noaa2)

  radius = 6371  ;earth mean radius, km
  cos_distance = cos(lat_noaa1/180.*!pi)*cos(lat_noaa2/180.*!pi)*$
		 cos(lon_noaa1/180.*!pi-lon_noaa2/180.*!pi)+$
		 sin(lat_noaa1/180.*!pi)*sin(lat_noaa2/180.*!pi)
  ;cos_distance might larger than 1 or less than -1, and lead arithmetic error: Floating illegal operand
  id_plus_1 =where(cos_distance gt 1, num_plus_1)
  id_minus_1=where(cos_distance lt -1, num_minus_1)
  if num_plus_1 gt 0  then cos_distance(id_plus_1)=1
  if num_minus_1 gt 0 then cos_distance(id_minus_1)=-1

  distance = radius*acos(cos_distance)

  id = where( dif_BT lt cri_bt and $
	      dif_time lt cri_time and $
	      distance lt cri_dist and $
	     abs(lza_noaa1) lt cri_sat and $
	     abs(lza_noaa2) lt cri_sat and $
	     abs(lza_noaa1-lza_noaa2) lt cri_sat , ncount)
  if ncount eq 0 then goto,next_month

  ;
  ; search the corresponding Rc, Rw, Cc, Cw, Ce 
  ;----------------------------------------------
  k = -1.
  search_id_1_big = fltarr(ncount)
  search_id_2_big = fltarr(ncount)
  match_id = fltarr(ncount)
  for jj = 0, ncount-1 do begin
    search_id_1 = where(sec_1  eq sec_noaa1(id(jj)) and doy_1   eq doy_match(id(jj)) ,search_num_1)
    search_id_2 = where(sec_2  eq sec_noaa2(id(jj)) and doy_2   eq doy_match(id(jj)) ,search_num_2)

    if search_num_1 ge 1 and search_num_2 ge 1 then begin
      k = k+1
      search_id_1_big[k] = search_id_1[search_num_1-1]
      search_id_2_big[k] = search_id_2[search_num_2-1]
      match_id[k] = id[jj]
    endif
  endfor
  search_id_1_big = search_id_1_big[0:k]
  search_id_2_big = search_id_2_big[0:k]
  match_id = match_id[0:k]

  ;
  ; equation of the planck function
  ;----------------------------------
  plcoef1 = double(1.191042869E-05)
  plcoef2 = double(1.438777)

  temp_noaa1 = total(bt_digital_1(*,search_id_1_big),1)/total(finite(bt_digital_1(*,search_id_1_big)),1)
  temp_noaa2 = total(bt_digital_2(*,search_id_2_big),1)/total(finite(bt_digital_2(*,search_id_2_big)),1)

  id = where(temp_noaa2 gt 0 and temp_noaa1 gt 0)
  search_id_1_big = search_id_1_big[id]
  search_id_2_big = search_id_2_big[id]
  match_id = match_id[id]

  temp_noaa1 = temp_noaa1[id]
  temp_noaa2 = temp_noaa2[id]

  temp_cold = 4.78 ; Kelvin degree

  c1v3_noaa1 = plcoef1*pl_noaa1(chan-1)*pl_noaa1(chan-1)*pl_noaa1(chan-1);
  c2v_noaa1  = plcoef2*pl_noaa1(chan-1);

  c1v3_noaa2 = plcoef1*pl_noaa2(chan-1)*pl_noaa2(chan-1)*pl_noaa2(chan-1);
  c2v_noaa2  = plcoef2*pl_noaa2(chan-1);

  Rw1 = c1v3_noaa1/(exp(c2v_noaa1/temp_noaa1)-1)
  Rw2 = c1v3_noaa2/(exp(c2v_noaa2/temp_noaa2)-1)
  Rc1 = c1v3_noaa1/(exp(c2v_noaa1/temp_cold)-1)
  Rc2 = c1v3_noaa2/(exp(c2v_noaa2/temp_cold)-1)

  ;
  ; Set seached count
  ;----------------------------------
  Cw1=count_warm_1[search_id_1_big]
  Cw2=count_warm_2[search_id_2_big]
  Cc1=count_cold_1[search_id_1_big]
  Cc2=count_cold_2[search_id_2_big]
  Ce1=count_scene_1[search_id_1_big]
  Ce2=count_scene_2[search_id_2_big]

  ;
  ; Set saved data
  ;----------------------------------
  t = fltarr(n_elements(match_id))
  tdata = transpose([[year[match_id]],[month_loop+t],$
	      [doy_match[match_id]],[sec_noaa1[match_id]],[sec_noaa2[match_id]],$
	      [lat_noaa1[match_id]],[lon_noaa1[match_id]],[lat_noaa2[match_id]],[lon_noaa2[match_id]],$
	      [Rw1[*]],[Rw2[*]],[Rc1+t],[Rc2+t],[Cw1[*]],[Cw2[*]],[Ce1[*]],[Ce2[*]],[Cc1[*]],[Cc2[*]],$
	      [bt_noaa1[match_id]],[bt_noaa2[match_id]] ])
  if n_elements(data) eq 0 then begin
    data = tdata
  endif else begin
    data = [[data],[tdata]]
  endelse
  next_month:
endfor

;
; Save data
;----------------------------------
save,filename=outfile,data

pro_end:
stop
end 

