pro sno_step2_soluteequation_msu
;=====================================================================
;@***h*  MW_Sounder_FCDR/sno_step2_soluteequation_msu.pro
; 
; NAME
;        sno_step2_soluteequation_msu.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the saved matched MSU raw counts data, this program 
;              loops over the names of NOAA missions to calculate the 
;              slopes and offsets for all the missions saved to ASCII 
;              files for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Matched NOAA coefficient data:
;                $InPath/step1_{noaa1}_{noaa2}.sav
;
; OUTPUTS:     
;              Generate NOAA radiance offset and nonlinear coefficients
;                $OutPath/*.dat
;
; AUTHOR:      
;              Liang Peng / Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;          jd  = julian day
; InPath_coef  = Path to monthly fit coeffcients.
; InPath_match = Path to matched polar orbiter data
; OutPath      = Path to output data
;
;              Path values should contain a trailing '/'
;**************************************************************************
InPath  = '../Data/tmp/sno/match_coef/'
OutPath = '../Data/tmp/sno/match_coef/'
noaa2s = ['noaa14/noaa12','noaa12/noaa11','noaa11/noaa10']
up = 0.
down = -10.
;*********************************************************
; End of RUN parameters
;*********************************************************

; 
; Loop over the NOAA missions to process
;----------------------------------
for inoaa = 0,n_elements(noaa2s)-1 do begin

  ;
  ; Set NOAA missions to process
  ;---------------------------------- 
  noaasi = strsplit(noaa2s[inoaa],'/',/extract)
  noaa2 = noaasi[0]
  noaa1 = noaasi[1]
 
  ;
  ; Read in offset and nonlinear coefficients of noaa2,
  ; NOAA14's offset are set 0, nonlinear coefficients are set from 'down' to 'up'
  ;---------------------------------- 
  infile_coef  = Inpath+'step2_'+noaa2+'_coef.dat'
  if inoaa eq 0 then begin
    data = fltarr(2,11)
    data[1,*] = findgen(11)/10*(up-down)+down
    close,2
    openw,2,infile_coef
    for idata = 0,10 do begin
      printf,2,data[*,idata],format='(2(f12.6,x))'
    endfor
    close,2
  endif else begin
    data = fltarr(2,11)
    close,2
    openr,2,infile_coef
    readf,2,data
    close,2
  endelse

  ;
  ; Set offset and nonlinear coefficients of noaa2
  ;---------------------------------- 
  offset_1    = data(0,*)*1E-5
  nonlinear_1 = data(1,*)
  total_coef = n_elements(offset_1)

  ;
  ; Set output file name
  ;---------------------------------- 
  outfile_coef  = Outpath+'step2_'+noaa1+'_coef.dat'
  print,outfile_coef

  ; 
  ; Set input file name and read in matched coefficients
  ;----------------------------------
  infile_match = Inpath+'step1_'+noaa1+'_'+noaa2+'.sav'
  restore,infile_match
  data_total = data

  ; 
  ; Select good id
  ;----------------------------------
  m18 = 800 
  if noaa2 eq 'noaa14' then begin  ;noaa12 and noaa14
    count_valid_id = where(data_total[13,*] gt data_total[15,*] and $
			   data_total[15,*] gt data_total[17,*] and $
			   data_total[14,*] gt data_total[16,*] and $
			   data_total[16,*] gt data_total[18,*] and $
			   data_total[18,*] lt m18 ,count_valid_num)   ;Cc_noaa14 have one wrong pixel
  endif else if noaa1 eq 'noaa11' then begin  ;noaa11 and noaa2
    count_valid_id = where(data_total[13,*] gt data_total[15,*] and $
			   data_total[15,*] gt data_total[17,*] and $
			   data_total[14,*] gt data_total[16,*] and $
			   data_total[16,*] gt data_total[18,*] and $
			   data_total[13,*] lt 3500 ,count_valid_num)   ;Cw_noaa11 have one wrong pixel
  endif else begin
    count_valid_id = where(data_total[13,*] gt data_total[15,*] and $     ;Cw > Ce > Cc
			   data_total[15,*] gt data_total[17,*] and $
			   data_total[14,*] gt data_total[16,*] and $
			   data_total[16,*] gt data_total[18,*] $
			    ,count_valid_num)   
  endelse

  ; 
  ; Set radiance values
  ;---------------------------------- 
  Rw2 = data_total[9, count_valid_id]
  Rw1 = data_total[10,count_valid_id]
  Rc2 = data_total[11,count_valid_id]
  Rc1 = data_total[12,count_valid_id]
  Cw2 = data_total[13,count_valid_id]
  Cw1 = data_total[14,count_valid_id]
  Ce2 = data_total[15,count_valid_id]
  Ce1 = data_total[16,count_valid_id]
  Cc2 = data_total[17,count_valid_id]
  Cc1 = data_total[18,count_valid_id]
  Tb2 = data_total[19,count_valid_id]
  Tb1 = data_total[20,count_valid_id]

  ; 
  ; Set linear item and nonlinear item
  ;---------------------------------- 
  S1  = (Rw1-Rc1)/(Cw1-Cc1)
  S2  = (Rw2-Rc2)/(Cw2-Cc2)
  Z1  = S1^2*(Ce1-Cc1)*(Ce1-Cw1) 
  Z2  = S2^2*(Ce2-Cc2)*(Ce2-Cw2) 
  Rl1 = Rc1+S1*(Ce1-Cc1) 
  Rl2 = Rc2+S2*(Ce2-Cc2) 

  ; 
  ; Generate the offset and nonlinear coefficients of noaa1 and save
  ;---------------------------------- 
  close,2
  openw,2,outfile_coef
  for coef_loop = 0, total_coef-1 do begin
    R1= Rl1-Rl2-offset_1(coef_loop)+nonlinear_1(coef_loop)*Z1
    nonlinear_2 = total( (Z2-mean(Z2))*(R1-mean(R1)) )/total( (Z2-mean(Z2))^2  )
    offset_2 = -( mean(R1) - nonlinear_2*mean(Z2)  )
    printf,2,[offset_2*1E5,nonlinear_2],format='(2(f12.6,x))'
  endfor
  close,2
endfor
pro_end:
stop
end 

