pro sno_step3_rebuild_msu
;=====================================================================
;@***h*  MW_Sounder_FCDR/sno_step3_rebuild_msu.pro
; 
; NAME
;        sno_step3_rebuild_msu.pro
; 
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              This program loops over the specified range of Julian days
;               and years, and reads in MSU raw count and slopes and offsets
;               to rebuild BTs written to an NetCDF data file for later use.
;
; 
; EXTERNALS:   
;              msu_parameter.pro
; 
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
; 
; INPUTS:      
;              Original NOAA coefficients:
;                $InPath_rad/
;
;              Offset and Nonlinear coefficients:
;                $InPath_coef/
; 
; OUTPUTS:     
;              Rebuilt NOAA MSU Brightness Temperatures 
;                $OutPath_noaa/
; 
; AUTHOR:      
;              Liang Peng / Xinjia Zhou / Ben Ho (?)
; 
; REVISIONS:   
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;       JD_BEG = The begining Julian day to process
;       JD_END = The ending Julian day to process
;         chan = MSU channel to process 
;         noaa = Mission name to process
; InPath_rad   = Path to original MSU radiance data
; InPath_coef  = Path to monthly fit coeffcients.
; OutPath      = Path to output data
; lzamax       = Maximum satellite zenith angle
;
;              Path values should contain a trailing '/'
;**************************************************************************
      JD_BEG = julday(1,1,2001)
      JD_END = julday(1,1,2001)
      chan   = 3
      noaa   = 'noaa6'
InPath_rad   = '../Data/tmp/extract/msu_coef/'
InPath_coef  = '../Data/tmp/sno/match_coef/'
OutPath      = '../Data/tmp/sno/rebuild/orbit/'
lzamax       = 30
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set Julian days that will be dealwithed
;-----------------------------------------------------------------------
jds = timegen(start=JD_BEG,final=JD_END,unit='month')

;
; Loop over the Julian days to process
;----------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set the month and year
  ;----------------------------------
  jd = jds[ijd]
  caldat,jd,month_loop,dom_loop,year_loop
  print,'------------------------'
  print,year_loop,month_loop,noaa
  print,'------------------------'
  jdst = julday(month_loop,1,year_loop)
  jdnd = julday(month_loop+1,0,year_loop)

  ;
  ; Set scan id
  ;----------------------------------
  scan_position_name = [-47.35,-37.88,-28.41,-18.94,-9.47,0,9.47,18.94,28.41,37.88,47.35]
  scan_position_id = where(abs(scan_position_name) lt lzamax)
  nscan = n_elements(scan_position_id)

  ;
  ; Set NOAA parameters
  ;----------------------------------
  msu_parameter,noaa,pl_noaa,k_noaa,e_1a_noaa,e_2a_noaa,e_1b_noaa,e_2b_noaa

  ;
  ; Read in the offset and nonlinear coefficients
  ;----------------------------------
  infile_coef = InPath_coef+'step2_'+noaa+'_coef.dat'
  data_total = fltarr(2,11)
  close,2
  openr,2,infile_coef
  readf,2,data_total
  close,2
  offset    = data_total[0,*]*1E-5
  nonlinear = data_total[1,*]

  ;
  ; Read in MSU coef data
  ;---------------------------------
  var = ['year','doy','sec','alt','lat','lon','count','digital','coef','flag']
  infile_rad = Inpath_rad+str_jd(jd,'yyyymm')+'_msu_'+noaa+'_nc'
  if ~file_test(infile_rad) then goto,next_month
  ncid = NCDF_OPEN(infile_rad,/NOWRITE)
  for iv = 0,n_elements(var)-1 do begin
    vd = NCDF_VARID(ncid, var[iv])
    rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv])
  endfor
  NCDF_CLOSE, ncid
  lat = lat(scan_position_id,*)
  lon = lon(scan_position_id,*)

  ;
  ; Set count
  ;---------------------------------
  count_scene = count(scan_position_id*4+chan-1,*)
  count_cold  = count(43+chan,*) ;space viewing
  count_warm = count(47+chan,*) ;warm target 

  ;
  ; Set digital
  ;---------------------------------
  num_coef = n_elements(doy)
  bt_digital = dblarr(2,num_coef) - 999.
  count_target  = digital(16:17,*) ; target2 (for ch3 and ch4)
  count_low     = digital(1,*)     ; low reference 
  count_high    = digital(3,*)     ; high reference
  e_a_noaa = e_2a_noaa
  e_b_noaa = e_2b_noaa

  ;
  ; Check data
  ;---------------------------------
  divident_zero_id = where(count_high ne count_low, divident_zero_num)
  if divident_zero_num eq 0 then goto,next_month
  Ra_noaa = k_noaa(0) + k_noaa(1)*[count_target(0,divident_zero_id)-count_low(divident_zero_id)]/[count_high(divident_zero_id)-count_low(divident_zero_id)]  ;1st warm scan
  Rb_noaa = k_noaa(0) + k_noaa(1)*[count_target(1,divident_zero_id)-count_low(divident_zero_id)]/[count_high(divident_zero_id)-count_low(divident_zero_id)]  ;2nd warm scan

  bt_digital(0,divident_zero_id) = Ra_noaa * e_a_noaa(2)^2 + Ra_noaa * e_a_noaa(1) + e_a_noaa(0)
  bt_digital(1,divident_zero_id) = Rb_noaa * e_b_noaa(2)^2 + Rb_noaa * e_b_noaa(1) + e_b_noaa(0)

  ;
  ; Set coef
  ;---------------------------------
  Calibration_Coefficients_normalization = coef(12:15,*)
  Calibration_Coefficients = coef(22:23,*)
   
  ;
  ; Set flag     
  ;---------------------------------
  scan_quality_indicator = byte(flag(0:2,*))  ; only first 3 byte contain flag, the 4th contain count
  Scan_Position_Quality  = byte(flag(4+scan_position_id,*)) ; 11:21-7 is 11 earth scan positions, 22-7 is cold scan, 23-7 is warm scan, 24-7 is scan to spot, 25:26-7 are last two uesless columns 

  ;
  ; Get the average of the warm target temperature ---
  ;---------------------------------
  total_num_digital = n_elements(sec)
  bt_digital_mean = dblarr(total_num_digital)-999.
  jj = 0L
  while jj lt total_num_digital do begin
    valid_bt_digital_id = where(bt_digital(*,jj) gt 0 , valid_bt_digital_num)
    if valid_bt_digital_num gt 0 then bt_digital_mean(jj) = mean(bt_digital(valid_bt_digital_id,jj))  
    jj = jj +1L
  endwhile
  bt_digital_mean = bt_digital_mean(0:jj-1)

  ;
  ; Remove the unvalid data
  ;---------------------------------
  msu_yrange,noaa,chan,yrange_count,yrange0,yrange1,coldrange
  doyst = julday(month_loop,1,year_loop)-julday(1,0,year_loop)
  doynd = julday(month_loop+1,0,year_loop)-julday(1,0,year_loop)

  if noaa eq 'tiros' and year_loop eq 1978 and (month_loop eq 10 or month_loop eq 11 ) then begin
    print,noaa, year_loop, month_loop
    valid_id = where(year eq year_loop and doy  ge doyst and doy le doynd and $
		     Calibration_Coefficients(0,*) ge yrange0(0) and Calibration_Coefficients(0,*) le yrange0(1) and $
		     Calibration_Coefficients(1,*) ge yrange1(0) and Calibration_Coefficients(1,*) le yrange1(1) and $
		     bt_digital_mean gt 0             and $
		     count_warm lt 3500               and $
		     count_cold lt coldrange[1]       and $
		     count_cold gt coldrange[0]       and $
		    (scan_quality_indicator(0,*) eq 0 or scan_quality_indicator(0,*) eq 1) and $  ;for tiros in 1978.10~11 only
		    (scan_quality_indicator(1,*) and 'FB'x) eq 0 and $ ; 'FB' is 11111011, than means only mirror sequence in byte 10 is neglected
		     scan_quality_indicator(2,*)            eq 0     $
		     , valid_num)
  endif else begin
    valid_id = where(year eq year_loop and doy  ge doyst and doy le doynd and $
		     Calibration_Coefficients(0,*) ge yrange0(0) and Calibration_Coefficients(0,*) le yrange0(1) and $
		     Calibration_Coefficients(1,*) ge yrange1(0) and Calibration_Coefficients(1,*) le yrange1(1) and $
		     bt_digital_mean gt 0             and $
		     count_warm lt 3500               and $
		     count_cold lt coldrange[1]       and $
		     count_cold gt coldrange[0]       and $
		    scan_quality_indicator(0,*)            eq 0 and $
		    (scan_quality_indicator(1,*) and 'FB'x) eq 0 and $ ; 'FB' is 11111011, than means only mirror sequence in byte 10 is neglected
		     scan_quality_indicator(2,*)            eq 0     $
		     , valid_num)
  endelse

  valid_id = where(year eq year_loop and doy  ge doyst and doy le doynd and $
		scan_quality_indicator(0,*)            eq 0 and $
		    (scan_quality_indicator(1,*) and 'FB'x) eq 0 and $ ; 'FB' is 11111011, than means only mirror sequence in byte 10 is neglected
		     scan_quality_indicator(2,*)            eq 0     $
		     , valid_num)       

  year      = year(valid_id)
  doy       = doy(valid_id)
  sec       = sec(valid_id)
  alt       = alt(valid_id)
  lat       = lat(*,valid_id)
  lon       = lon(*,valid_id)
  count_warm      = count_warm(valid_id)
  count_cold      = count_cold(valid_id)
  bt_digital_mean = bt_digital_mean(valid_id)
  Calibration_Coefficients_normalization = Calibration_Coefficients_normalization(*,valid_id)
  scan_quality_indicator = scan_quality_indicator(*,valid_id)
  Scan_Position_Quality  = Scan_Position_Quality(*,valid_id)
  count_scene     = count_scene(*,valid_id)


  for kk = 0,nscan-1 do begin ; nadir view position
    valid_id = where(count_scene(kk,*) ge yrange_count(0)    and $
		     count_scene(kk,*) le yrange_count(1)    and $
		     count_scene(kk,*) lt count_warm         and $
		     count_scene(kk,*) gt count_cold         and $
		    (Scan_Position_Quality(kk,*)  and 'FD'x) eq 0, $  ; 'FD' is 11111101, that means the mirror seqence in table 4.3.2.1-7 is neglectable
		     valid_num1,  COMPLEMENT=unvalid_id, NCOMPLEMENT=unvalid_num )
    
    if unvalid_num gt 0 then count_scene(kk,unvalid_id) =  -999.
  endfor

  ;
  ; Transfer from raw count to normalized count
  ;---------------------------------------------------
  count_normalized_scene     = fltarr(nscan,valid_num)-999.
  for scan_loop = 0,nscan-1 do begin ; nscan nadir view position
     valid_id = where(count_scene(scan_loop,*) gt 0 )
     raw_count = count_scene(scan_loop,valid_id)

     count_normalized_scene(scan_loop,valid_id) = Calibration_Coefficients_normalization(0,valid_id)            *double(2.^(-22))+$
						    Calibration_Coefficients_normalization(1,valid_id)*raw_count  *double(2.^(-30))+$
						    Calibration_Coefficients_normalization(2,valid_id)*raw_count^2*double(2.^(-44))+$
						    Calibration_Coefficients_normalization(3,valid_id)*raw_count^3*double(2.^(-56))
  endfor

  count_normalized_cold = Calibration_Coefficients_normalization(0,*)               *double(2.^(-22))+$
   	                  Calibration_Coefficients_normalization(1,*)*count_cold  *double(2.^(-30))+$
			  Calibration_Coefficients_normalization(2,*)*count_cold^2*double(2.^(-44))+$
			  Calibration_Coefficients_normalization(3,*)*count_cold^3*double(2.^(-56))

  count_normalized_warm = Calibration_Coefficients_normalization(0,*)               *double(2.^(-22))+$
			  Calibration_Coefficients_normalization(1,*)*count_warm  *double(2.^(-30))+$
			  Calibration_Coefficients_normalization(2,*)*count_warm^2*double(2.^(-44))+$
			  Calibration_Coefficients_normalization(3,*)*count_warm^3*double(2.^(-56))

  ;
  ; Equation of the planck function
  ;-------------------------------------------------
  plcoef1 = double(1.191042869E-05)
  plcoef2 = double(1.438777)

  temp_warm = bt_digital_mean
  temp_cold = 4.78 ; Kelvin degree

  c1v3_noaa = plcoef1*pl_noaa(chan-1)*pl_noaa(chan-1)*pl_noaa(chan-1);
  c2v_noaa  = plcoef2*pl_noaa(chan-1);

  rad_warm  = c1v3_noaa/(exp(c2v_noaa/temp_warm)-1)
  rad_cold  = c1v3_noaa/(exp(c2v_noaa/temp_cold)-1)

  ;
  ; Rebuild BT
  ;------------------------------------------------
  var = ['year','month','doy','sec','alt','lat','lon','spn','tbw','tb']
  for iv = 0,n_elements(var)-2 do begin
    rt = execute(var[iv]+'_rb = fltarr(nscan,n_elements(year))')
  endfor 
  tb_rb = fltarr(11,nscan,n_elements(year))
  dm = fltarr(nscan,n_elements(year))
  tbw = temp_warm;

  Rc =rad_cold
  Rw=rad_warm
  Cw=count_warm
  Cc=count_cold

  for scan_position_loop = 0,nscan-1 do begin
    Ce=count_scene[scan_position_loop,*]             ; raw earth count

    dmi = ce lt 0
    id = where(dmi)
    if id[0] gt -1 then ce[id] = 1
    dm[scan_position_loop,*] = dmi

    S = (Rw-Rc)/(Cw-Cc)  ;S is gain
    Z = S^2*(Ce-Cc)*(Ce-Cw)

    Re = fltarr(11,n_elements(S))
    for nonlinear_loop = 0,10 do begin
      Re(nonlinear_loop,*) = Rc+S*(Ce-Cc)-offset(nonlinear_loop)+nonlinear(nonlinear_loop)*Z
    endfor

    ;
    ; Transfer the radiance to Bt
    ;-----------------------------------------
    Tb = c2v_noaa/alog(double(c1v3_noaa/Re) + 1.0)
    spn = scan_position_name[scan_position_id[scan_position_loop]]
    month = month_loop

    for iv = 0,n_elements(var)-2 do begin
      if var[iv] eq 'lat' or var[iv] eq 'lon' then begin
	rt = execute(var[iv]+'_rb[scan_position_loop,*] = '+var[iv]+'[scan_position_loop,*]')
      endif else begin
	rt = execute(var[iv]+'_rb[scan_position_loop,*] = '+var[iv])
      endelse
    endfor
    tb_rb[*,scan_position_loop,*] = tb

    jump_Ce:
  endfor ; scan_position

  ;
  ; Change the dimensions
  ;-----------------------------------------
  num = n_elements(year_rb)
  for iv = 0,n_elements(var)-2 do begin
    rt = execute(var[iv]+'_rb = reform('+var[iv]+'_rb,num,1)')
  endfor

  tb_rb = reform(tb_rb,11,num)
  dm = reform(dm,num,1)
  id = where(~dm)
  for iv = 0,n_elements(var)-2 do begin
    rt = execute(var[iv]+'_rb = '+var[iv]+'_rb[id]')
  endfor
  tb_rb = tb_rb[*,id]

  ;
  ; Save data to netcdf format day by day
  ;----------------------------------------
  for jdm = jdst,jdnd do begin
    doyi = jdm - julday(1,0,year_loop)
    id = where(doy_rb eq doyi)

    if n_elements(id) lt 5 then goto,next_day

    ptrarr,ptr_var,n_elements(var)
    for iv = 0,n_elements(var)-2 do begin
      rt = execute('*ptr_var[iv] = '+var[iv]+'_rb[id]')
    endfor
    *ptr_var[n_elements(var)-1] = tb_rb[*,id]

    
    outfile = Outpath+str_jd(jd,'yyyymm')+'/'+str_jd(jdm,'yyyyddd')+'_msu_'+noaa+'_rebuild_nc'    
    outdir = file_dirname(outfile)
    if ~file_test(outdir) then spawn,'mkdir -p '+outdir
    print,outfile

    ;
    ; Write data to output file
    ;-------------------------------------
    fid = NCDF_CREATE(outfile,/CLOBBER)
    rebuild_dim = NCDF_DIMDEF(fid, 'rebuild_number', 11)
    time_dim = NCDF_DIMDEF(fid, 'time', n_elements(lat[id]))
    NCDF_CONTROL, fid, /FILL
    vid = lonarr(n_elements(var))
    for iv = 0,n_elements(var)-1 do begin
      type = size(*ptr_var[iv],/tname)
      if var[iv] ne 'tb' then begin
        rt=execute('vid[iv] = NCDF_VARDEF(fid, var[iv], time_dim,/'+type+')');
      endif else begin
        rt=execute('vid[iv] = NCDF_VARDEF(fid, var[iv], [rebuild_dim,time_dim],/'+type+')');
      endelse
    endfor
    NCDF_CONTROL, fid, /ENDEF
    for iv = 0,n_elements(var)-1 do begin
      NCDF_VARPUT, fid, vid[iv], *ptr_var[iv]
    endfor
    NCDF_CLOSE, fid ; Close the netCDF file.
    next_day:
  endfor

  next_month:
endfor

pro_end:
stop
end 

