@bin2lat6.pro
pro sno_step4_intersatbias_msu
;=====================================================================
;@***h*  MW_Sounder_FCDR/sno_step4_intersatbias_msu.pro
;
; NAME
;        sno_step4_intersatbias_msu.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              For specified range of Julian days, this program read in 
;              the MSU gridded rebuilt BTs and generate the intersatellite 
;              biases written to an IDL data file for later use.
;
; EXTERNALS:
;              bin2lat6.pro
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
;
; INPUTS:
;              Rebuilt NOAA brighness temperature:
;                $InPath_rebuild/
;
; OUTPUTS:
;              Inter satellite bias
;                $OutPath_noaa/
;
; AUTHOR:
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
; 
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;           JD_BEG = The begining Julian day to process
;           JD_END = The ending Julian day to process
;            noaas = Names of mission name to process
;           Inpath = Path to rebuild gridded data
;          OutPath = Path to output data
;
;              Path values should contain a trailing '/'
;**************************************************************************
JD_BEG = julday( 1, 1,1979)
JD_END = julday(12,31,2006)
noaas = 'noaa14/noaa12'
Inpath = '../Data/tmp/sno/rebuild/grid/'
Outpath = '../Data/tmp/sno/rebuild/bias/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set names of missions
; -----------------------------------
if n_elements(noaas) eq 1 then begin
  noaas = strsplit(noaas,'/',/extract)
endif
noaa1 = noaas[0]
noaa2 = noaas[1]

;
; set output file name
; -----------------------------------
Outfile = Outpath+noaa1+'-'+noaa2+'.sav'
 
;
; Set Julian days that will be dealwithed
;-----------------------------------------------------------------------
jds = timegen(start=JD_BEG,final=JD_END,unit='day')

bias = fltarr(6,n_elements(jds),11)+!values.d_nan 
tb_noaa1 = fltarr(6,n_elements(jds),11)+!values.d_nan
tb_noaa2 = fltarr(6,n_elements(jds),11)+!values.d_nan
tw_noaa1 = fltarr(6,n_elements(jds),11)+!values.d_nan
tw_noaa2 = fltarr(6,n_elements(jds),11)+!values.d_nan

;
; Loop over the Julian days to process
;----------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set the month and year
  ;----------------------------------
  jd = jds[ijd]
  caldat,jd,month_loop,dom_loop,year_loop
  doy_loop = jd - julday(1,0,year_loop)
  print,'------------------------'
  print,year_loop,month_loop,dom_loop
  print,'------------------------'

  ;
  ; Set input file names
  ;----------------------------------
  inol = findgen(11)
  var = ['tbbin'+string(inol,f='(i2.2)'),'tbwbin']
  infile1 = Inpath+string(year_loop,f='(i4.4)')+string(month_loop,f='(i2.2)')+'/'+$
            string(year_loop,f='(i4.4)')+string(doy_loop,f='(i3.3)')+'_'+noaa1+'_nc'
  infile2 = Inpath+string(year_loop,f='(i4.4)')+string(month_loop,f='(i2.2)')+'/'+$
            string(year_loop,f='(i4.4)')+string(doy_loop,f='(i3.3)')+'_'+noaa2+'_nc'

  ;
  ; Read in first warm target temperature
  ; ---------------------------------
  if file_test(infile1) and n_elements(tw0_noaa1) eq 0 then begin
    ncid = NCDF_OPEN(infile1,/NOWRITE)
    vd = NCDF_VARID(ncid, 'tbwbin')
    NCDF_VARGET, ncid,  vd, tbwbin
    NCDF_CLOSE, ncid
    id = where(tbwbin lt 50)
    if id[0] gt -1 then tbwbin[id] = !values.d_nan
    bin2lat6,tbwbin,tw0_noaa1
  endif
  if file_test(infile2) and n_elements(tw0_noaa2) eq 0 then begin
    ncid = NCDF_OPEN(infile2,/NOWRITE)
    vd = NCDF_VARID(ncid, 'tbwbin')
    NCDF_VARGET, ncid,  vd, tbwbin
    NCDF_CLOSE, ncid
    id = where(tbwbin lt 50)
    if id[0] gt -1 then tbwbin[id] = !values.d_nan
    bin2lat6,tbwbin,tw0_noaa2
  endif

  ;
  ; Read in brightness temperature
  ; ---------------------------------
  if ~(file_test(infile1) and file_test(infile2)) then goto,next
  ncid = NCDF_OPEN(infile1,/NOWRITE)
  for iv = 0,n_elements(var)-1 do begin
    vd = NCDF_VARID(ncid, var[iv])
    rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv]+'_noaa1')
  endfor
  NCDF_CLOSE, ncid

  ncid = NCDF_OPEN(infile2,/NOWRITE)
  for iv = 0,n_elements(var)-1 do begin
    vd = NCDF_VARID(ncid, var[iv])
    rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv]+'_noaa2')
  endfor
  NCDF_CLOSE, ncid

  ;
  ; Caculate the bias
  ; ---------------------------------
  for inol = 0,10 do begin
    rt=execute('tbbin_noaa1 = '+'tbbin'+string(inol,f='(i2.2)')+'_noaa1')
    rt=execute('tbbin_noaa2 = '+'tbbin'+string(inol,f='(i2.2)')+'_noaa2')
    
    id = where(tbbin_noaa1 lt 50 or tbbin_noaa2 lt 50)
    if id[0] gt -1 then begin 
      tbbin_noaa1[id] = !values.d_nan
      tbbin_noaa2[id] = !values.d_nan
    endif

    dbt = tbbin_noaa1-tbbin_noaa2
    bin2lat6,dbt,dlat6
    bias[*,ijd,inol] = dlat6

    bin2lat6,tbbin_noaa1,tblat_noaa1
    bin2lat6,tbbin_noaa2,tblat_noaa2

    tb_noaa1[*,ijd,inol] = tblat_noaa1
    tb_noaa2[*,ijd,inol] = tblat_noaa2
  endfor

  tbwbin_noaa1[id] = !values.d_nan
  tbwbin_noaa2[id] = !values.d_nan
  bin2lat6,tbwbin_noaa1,twlat_noaa1
  bin2lat6,tbwbin_noaa2,twlat_noaa2
  tw_noaa1[*,ijd] = twlat_noaa1
  tw_noaa2[*,ijd] = twlat_noaa2

  next:
endfor

;
; Save data
; -----------------------------------
jds_bias = jds
save,filename=outfile,tw0_noaa1,tw0_noaa2,tw_noaa1,tw_noaa2,tb_noaa1,tb_noaa2,bias,jds_bias

end
