pro sno_step6_chrisyprod_msu
;=====================================================================
;@***h*  MW_Sounder_FCDR/sno_step6_chrisyprod_msu.pro
;
; NAME
;        sno_step6_chrisyprod_msu.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              For specified range of Julian days and names of MSU missions, 
;              this program read in the MSU gridded rebuilt BTs and Christy 
;              coefficients to generate the calibrated MSU BTs for later use.
;
; EXTERNALS:
;              none.
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
;
; INPUTS:
;              Monthly matched NOAA MSU data:
;                $InPath_match/
;
;              Monthly NOAA coefficient data:
;                $InPath_coef/
;
; OUTPUTS:
;              Converted NOAA AMSU values are written to the
;              ASCCI file:
;                $OutPath_noaa/bin_{noaa}_YYYYMM_chan9_converted.dat
;
; AUTHOR:
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;      JD_BEG  = The Begining Julian day to process;julday(month,dayofmonth,year)
;      JD_END  = The Ending Julian day to process
; InPath_coef  = Path to coeffcients.
; InPath_grid  = Path to grid BT data
; OutPath      = Path to output data
;        noaa  = The polar orbiter to use
;        inol  = Number of guess
;
;              Path values should contain a trailing '/'
;**************************************************************************
JD_BEG      = julday( 1, 1,1979);
JD_END      = julday(12,31,2006)
InPath_coef = "../Data/tmp/sno/rebuild/coef/"
InPath_grid = "../Data/tmp/sno/rebuild/grid/"
OutPath     = "../Data/tmp/sno/rebuild/product/"
noaa        = 'noaa14'
inol = 0
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; string of guess number
;-----------------------------------------------------------------------
snol = string(inol,format='(i2.2)')

;
; Set Julian days that will be dealwithed
;-----------------------------------------------------------------------
jds = timegen(start=JD_BEG,final=JD_END,unit='day')

;
; Set Latitude scopes
;-----------------------------------------------------------------------
latlim1 = [-90,60,20,-20,-60,-90]
latlim2 = [ 90,90,60, 20,-20,-60]
latgd = findgen(72)*2.5-90+1.25
ilat = 0
restore,InPath_coef+'season_bias_lat'+string(ilat+1,f='(i2.2)')+'_nol'+snol+'.sav';mb

;
; generate the correction coefficients
; -----------------------------------
var = ['tbbin'+snol,'num']

; 
; Read in calibration coefficients
; -----------------------------------
ilat = 0
slat = 'lat'+string(ilat,f='(i2.2)')
cr = fltarr(4)
fc = Inpath_coef+'christy_ro_coeffient_'+slat+'_nol'+snol+'.dat'
ln = file_lines(fc)
close,2 & openr,2,fc
for i = 0,ln-1 do begin
  t = ''
  readf,2,t
  t = strsplit(t,/extract)
  if t[0] eq noaa then break
endfor
cr = double(t[1:4])
close,2

;
; Loop over the Julian days to process
;----------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set time variables
  ;----------------------------------
  jd = jds[ijd]
  caldat,jd,month,dom,year
  doy = jd-julday(1,0,year)
  adoy = doy*1.0/(julday(12,31,year)-julday(1,0,year))   
  syear  = string(year, f='(i4.4)')
  smonth = string(month,f='(i2.2)')
  sdoy   = string(doy,  f='(i3.3)')

 
  ;
  ; Set the infile
  ;---------------------------------- 
  Infile = Inpath_grid+syear+smonth+'/'+syear+sdoy+'_'+noaa+'_nc'
  if ~file_test(Infile)  then goto,next_day
  print,Infile

  ;
  ; Set the output file
  ;----------------------------------
  outfile = Outpath+syear+smonth+'/'+syear+sdoy+'_'+noaa+'_nol'+snol+'_nc'
  print,outfile

  ;
  ; Read in data
  ;----------------------------------
  var = ['tbbin'+snol,'num','tbwbin']
  ncid = NCDF_OPEN(Infile,/NOWRITE)
  for iv = 0,n_elements(var)-1 do begin
    vd = NCDF_VARID(ncid, var[iv])
    rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv])
  endfor
  NCDF_CLOSE, ncid
  rt=execute('tbbin=tbbin'+snol)

  ;
  ; Calibrate data
  ;----------------------------------
  chn_bin = tbbin+!values.d_nan
  chn_bin = tbbin-cr[1]*(tbwbin-cr[0])+cr[2]-cr[3]*(jd-julday(1,1,1980))

  idn = where(chn_bin le 0)
  if idn[0] gt -1 then chn_bin[idn] = -999

  ;
  ; Write data to output file
  ;-------------------------------------
  outdir = file_dirname(outfile)
  if ~file_test(outdir) then spawn,'mkdir -p '+outdir
  var = ['chn_bin','num']
  fid = NCDF_CREATE(outfile,/CLOBBER)
  lon_dim = NCDF_DIMDEF(fid, 'lon', 144)
  lat_dim = NCDF_DIMDEF(fid, 'lat', 72 )
  NCDF_CONTROL, fid, /FILL
  vid = lonarr(n_elements(var))
  for iv = 0,n_elements(var)-1 do begin
    rt=execute('type = size('+var[iv]+',/tname)')
    rt=execute('vid[iv] = NCDF_VARDEF(fid, var[iv], [lon_dim,lat_dim],/'+type+')');
  endfor
  NCDF_CONTROL, fid, /ENDEF
  for iv = 0,n_elements(var)-1 do begin
    rt=execute('NCDF_VARPUT, fid, vid[iv], '+var[iv])
  endfor
  NCDF_CLOSE, fid ; Close the netCDF file.
  next_day:
endfor

stop
end
