         program amsu_raob

!=====================================================================
!@***h*  MW_Sounder_FCDR/amsu_raob.f
!
! NAME:        
!   amsu_raob.exe
!
! LOCATION:    
!   $SRC_DIR/Src_fortran/
!
! DESCRIPTION: 
!              This program loops over the specified range of julian days 
!              and years, and reads in all of the RAOB profiles for each day.
!              Each profile is passed to an AMSU forward model routine
!              which calculates the brightness temperatures for 15
!              AMSU channels. The resulting brightness temperatures for
!              each day are written to an ASCII file for later use.
!
! EXTERNALS:   
!              AMSUfast module for AMSU Forward Model Calculations.
!
! USAGE:       
!              The user needs to first edit the RUN parameters for the program 
!              contained in the file 'CONFIG.amsu_raob' to set the range of 
!              days to process and the input/output paths for the data.
!
!              The program is then compiled to generate an executable:
!
!                   bash> make amsu_raob.exe
!                   bash> vi CONFIG.amsu_raob
!                   bash> ./amsu_raob.exe
!
! PARAMETERS:
!                 YEAR_BEG = Begining year
!                 JDAY_BEG = Begining julian day
!                 YEAR_END = Ending year
!                 JDAY_END = Ending julian day
!                   InPath = Path to interpolated GPSRO profiles
!                  OutPath = Path to output brightness temperature files
!
! INPUTS:      
!              Input ASCII files containing the RAOB profile data should be 
!              stored with the following directory structure and file format:
!
!                    $InPath/YYYY/Prf_YYYY.DDD.dat'
!
! OUTPUTS:     
!              AMSU brightness temperatures are stored in the files:
!
!                    $OutPath/YYYY/AMSU_raob_Bt_YYYY.DDD.dat
!
! AUTHOR:      
!              ??
!
! REVISIONS:   
!              Patrick Callaghan 
!               *Cleaned up and Packaged the AMSU Forward model routines 
!                 into module AMSUfast_mod.f for CDR
!               *Cleaned up, added program inputs via CONFIG file, and 
!                 Formatted main program for CDR
!@*****
!=====================================================================
         use AMSUfast

         parameter (nlev=100)
         character*100  infile,outfile,indir,outdir
         character*4    year_str
         character*3    day_str

         integer year, day
         integer ratp,corn,sirc,level_num
         integer year_loop,day_loop
         integer k,kanal
         integer LUI,iout1
         integer ilev,sfc_lev
         integer inum(nlev)
         integer nn

         real xlat,xlon
         real level_low,level_high,time
         real surface_press,surface_temp,emiss_mw
         dimension pres(nlev)
         dimension ta(nlev),wa(nlev)
         dimension wv(nlev),swv(nlev),rh(nlev)
         dimension tt(nlev,15), pres_raw(250),temp_raw(250)
         dimension ps(nlev)

         real taut(100)
         real amsu_tb(15),tt_chan(15),tt_chan_all(15)
         real T_500_300

         data ps   / 0.0050,    0.0161,    0.0384,    0.0769,    0.1370,
     &    0.2244,    0.3454,    0.5064,    0.7140,    0.9753,    1.2972,
     &    1.6872,    2.1526,    2.7009,    3.3398,    4.0770,    4.9204,
     &    5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492,
     &   14.4559,   16.4318,   18.5847,   20.9224,   23.4526,   26.1829,
     &   29.1210,   32.2744,   35.6505,   39.2566,   43.1001,   47.1882,
     &   51.5278,   56.1260,   60.9895,   66.1253,   71.5398,   77.2396,
     &   83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775,
     &  125.6456,  133.8462,  142.3848,  151.2664,  160.4959,  170.0784,
     &  180.0183,  190.3203,  200.9887,  212.0277,  223.4415,  235.2338,
     &  247.4085,  259.9691,  272.9191,  286.2617,  300.0000,  314.1369,
     &  328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738,
     &  424.4698,  441.8819,  459.7118,  477.9607,  496.6298,  515.7200,
     &  535.2322,  555.1669,  575.5248,  596.3062,  617.5112,  639.1398,
     &  661.1920,  683.6673,  706.5654,  729.8857,  753.6275,  777.7897,
     &  802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236,
     &  958.5911,  986.0666,  990.9476, 995.2319, 1008.170/

      ! Program Inputs
      !------------------
      integer      YEAR_END,YEAR_BEG
      integer      JDAY_END,JDAY_BEG
      character*80 InPath,OutPath
      integer      InPlen,OutPlen
      namelist /inputs/ YEAR_END,YEAR_BEG,
     &                  JDAY_END,JDAY_BEG,
     &                  InPath,OutPath

!---------------------------------------------------------
! initialization
!---------------------------------------------------------
      ! Set default values
      !--------------------
      YEAR_BEG=9999
      YEAR_END=2014
      JDAY_BEG=9999
      JDAY_END=9999
      InPath='NOPATH'
      OutPath='NOPATH'

      ! Read input values from CONFIG file
      !-------------------------------------
      open(unit=14,file='CONFIG.amsu_raob',status='old')
      read(14,inputs)
      close(14)

      ! Check that iniput values were specified in CONFIG file
      !-------------------------------------------------------
      if((    YEAR_BEG.eq.9999    ).or.(YEAR_END.eq.9999).or.
     &   (    JDAY_BEG.eq.9999    ).or.(JDAY_END.eq.9999).or.
     &   ( InPath(1:6).eq.'NOPATH').or.
     &   (OutPath(1:6).eq.'NOPATH')) then
        print *,' Not all program inputs were specified in CONFIG file:'
        print *,'      YEAR_BEG=',YEAR_BEG
        print *,'      YEAR_END=',YEAR_END
        print *,'      JDAY_BEG=',JDAY_BEG
        print *,'      JDAY_END=',JDAY_END
        print *,'        InPath=',InPath
        print *,'       OutPath=',OutPath
        stop
      endif

      ! Set string lengths, other fixed paramters
      !--------------------------------------------
      InPlen=len_trim(InPath)
      OutPlen=len_trim(OutPath)
      IUNIT=15
      OUNIT=16
      ilev = 100
      sfc_lev = 100

!---------------------------------------------------------
! Process Profiles
!---------------------------------------------------------
      ! Loop over the years to process
      !---------------------------------
      do year_loop=YEAR_BEG,YEAR_END

        ! Set the number of Julian days for this year
        !---------------------------------------------
        if((year_loop-2000).eq.(4*((year_loop-2000)/4))) then
          Njday=366
        else
          Njday=365
        endif

        ! Set the range of Julian days for the current year
        !---------------------------------------------------
        day_start=1
        day_end  =Njday
        if(year_loop.eq.YEAR_BEG) day_start=JDAY_BEG
        if(year_loop.eq.YEAR_END) day_end  =JDAY_END

        ! Loop over Julian days
        !------------------------
        do day_loop=day_start,day_end

          ! Open the Input/Output files
          !-----------------------------
          write(year_str,'(i4.4)') year_loop
          write(day_str ,'(i3.3)') day_loop

          infile =TRIM(InPath)//year_str//'/Prf_'//
     +             year_str//day_str//'.dat'
          outfile=TRIM(OutPath)//year_str//'/AMSU_raob_Bt_'//
     +             year_str//'.'//day_str//'.dat'
          print*,"output file=",outfile
          print*,"input file=",infile

          LUI = 15
          open(LUI,FILE=infile,STATUS='OLD',err=135)
         
          iout1 = 16
          open(iout1,FILE= outfile,STATUS='unknown')

          !----------------------------------------------------------------------------------------
          ! loop of each profile
          !----------------------------------------------------------------------------------------
          do while(.true.) ! end in 2000
            read(LUI,*,err=121,end=121)year,day,time,ratp,corn,sirc,
     +      xlat,xlon,level_num
            print*,year,day,time,ratp,corn,sirc,
     +      xlat,xlon,level_num
            do ii = 1,level_num
              read(LUI,*)lev,pres_raw(ii),temp_raw(ii)
            enddo 
            do ii = 1,100
              read(LUI,*)lev,pres(ii),ta(ii),wv(ii),swv(ii),inum(ii)
            enddo 
            do ii=1,100
              if(wv(ii).lt.0) then 
                wa(ii)=1e-5
              else if (wv(ii) .ge. pres(ii)) then
                wa(ii)=1000.
              else 
                wa(ii)=1000*wv(ii)/(pres(ii)-wv(ii))
              end if

              if(wv(ii) .ge. 0) then
                rh(ii) = 100.*wv(ii)/swv(ii)
              else
                rh(ii) = -9.
              endif
            enddo

            !-----------------------------------------------------------              
            ! Loop for channel AMSU cahnnel from kanal=1-15  
            !-----------------------------------------------------------              
            theta = 0.
            do kanal = 1,15
              call aqua_amsu_tran(ta,wa,theta,kanal,taut)
              surface_temp=300
              emiss_mw    =0
              call intrte_mws(taut,ta,surface_temp,emiss_mw,
     +                        sfc_lev,tbbd,tbbr,tbbs,tbbt)
              amsu_tb(kanal) = tbbt
              tt_chan(kanal)    =0
              tt_chan_all(kanal)=0
              do jj = 1,ilev-1
                tt(jj,kanal) =
     +          100*(taut(jj+1)-taut(jj))/(alog(ps(jj))-alog(ps(jj+1)))
              enddo
            enddo 
            
            write(iout1,2001) year,day,time,xlat,xlon,
     +                        (amsu_tb(k),k=1,15),
     +                        ratp,corn,sirc,level_num
2001        format(1x,i4,1x,i3,1x,f8.3,1x,2(1x,f7.2),
     +             2x,15(f7.2,1x),4(i4,1x))
            do ii = 1,level_num
              write(iout1,2002)ii,pres_raw(ii),temp_raw(ii)
            enddo
            inum(1) = 0
            do ii = 1,100
              write(iout1,2002)ii,ps(ii),ta(ii),rh(ii),inum(ii),
     +             (tt(ii,k),k=1,15)
2002          format(i3,2x,3(f8.3,2x),i3,2x,15(f9.3,2x))
            enddo
2000      enddo

121       close(LUI)              
          close(iout1)              

135       continue
        enddo
      enddo  
      stop 
!
! End Program
!---------------
      end 
